function [a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,tpa,pm,ctp,pv,s_vm,emv] = DSGE_MS_state_space_matrices_simple(gx,hx,gxx,hxx,gss,hss,eta,etastd,deltav)
% new version 04-27-2020, amended to allow A1 and C1 to have 
% different pages (if hx and gx have a third dimension). 
% Otherwise A1 and C1 have identical pages. 
% next two lines of comments are obsolete
% differs from DSGE_MS_state_space_matrices because solution matrices Gx,
% Gxx, Hx, Hxx do not depend on discrete states (as in case where MS
% regards only variances)
% inputs:  
%        gx,hx,gxx,hxx,gss,hss,etastd,deltav
% outputs
%        state equation matrices
%        a_i, A1_i, A2 B_i
%        measurement equation matrices
%        c_j, C1_j, C2, D; 
%        Sigma_zz approximation to ergodic covariance matrix of the z
%        process
%        discrete processes  
%        tpa transition probabilities array
%        pm  (2 ns) matrix with ergodic probs for each state
%        ctp (ncs x ncs) TPM for composite state
%        pv  (ncs x 1) vector with ergodic probs of composite state
%        s_vm (2 x ncs) matrix with possible values of composite state in dev from ergodic mean
%        emv = (ns x 1) vector with ergodic means
[ny,~]=size(gx); 
[nz,nshocks,ncs]=size(etastd);
ns=floor(log(ncs)/log(2));
nz2=nshocks;
%nz1=nz-nz2;
pim=reshape(deltav(1:ns*2),2,ns); % each col contains diagonal elements of TP for each discrete state
% compute ergodic probs and array with TPs (each page a different process)
pm=zeros(2,ns);
tpa=zeros(2,2,ns);
for is=1:ns
    pm(1,is)=(1-pim(2,is))/(2-sum(pim(:,is)));
    pm(2,is)=1-pm(1,is);
    tpa(:,:,is)=[pim(1,is) 1-pim(1,is); 1-pim(2,is)  pim(2,is)];    
end
% composite state tp matrix
ctp=tpa(:,:,ns);
for is=(ns-1):-1:1
    ctp=kron(tpa(:,:,is),ctp);
end
%composite state ergodic prob vector
pv=pm(:,ns);
for is=(ns-1):-1:1
    pv=kron(pm(:,is),pv);
end
% we can now draw from the ergodic distribution of the composite discrete %
% state variable
%csv_i=draw_multinom(pv,N,ncs,1);
% now get the moments of ergodic distribution for zetas
%construct Sigma_ss
%Sigma_ss=diag(prod(pm));
% now compute Sigma_zz
%need compute Hx_zz and Hx_zs
% Hx_zz=hx(1:nz,1:nz);
% Hx_zs=hx(1:nz,nz+1:end);
Omega_mean=zeros(nz,nz);
for is=1:ncs
     Omega_mean=Omega_mean+squeeze(eta(1:nz,1:nz,is))*pv(is);
end
Sigma_zz=Omega_mean;
%Sigma_zz=Hx_zs*Sigma_ss*Hx_zs'+Omega_mean;
%Sigma_zz=reshape((eye(nz^2)-kron(Hx_zz,Hx_zz))\Sigma_zz(:),nz,nz);
% next loop added 04-27-2020
if length(size(hx))==3
    hx_=zeros(nz,nz);
    for is=1:length(pv)
        hx_=hx_+hx(:,:,is)*pv(is);
    end
else
    hx_=hx;
end
% next instruction substituted on 04-27-2020
%Sigma_zz=VAR1_vcv(hx,Sigma_zz,1e+3,1e-8);
Sigma_zz=VAR1_vcv(hx_,Sigma_zz,1e+3,1e-8);

% initialise the z variables
%zm=(chol(Sigma_zz)')*randn(nz,N);
%% obtain indexes needed to linearise ME
% construction of these indexes checked
% this section needs to be executed only once and the 3 index vectors
% countxx=reshape((1:(nx^2))',nx,nx);
% index_xx_zz=vech(countxx(1:nz,1:nz)); 
% index_xx_zs=reshape(countxx(nz+1:end,1:nz)',ns*nz,1);
% index_xx_ss=vech(countxx(nz+1:end,nz+1:end));
%write matrices with all possible values for s and s .*. s
s_vm=zeros(ns,ncs);
for is=1:ns
    s_vm(is,:)=repmat([ones(1,ncs/(2^is)) zeros(1,ncs/(2^is))],1,2^(is-1));
end
emv=pm(1,:)'; % vector with ergodic means of the discrete states
s_vm=s_vm-repmat(emv,1,ncs);
%sq_vm=quadc1(s_vm);
%% compute blocks of the ssf matrices
% Gxx_zz=gxx(:,index_xx_zz);
% Gxx_zs=gxx(:,index_xx_zs);
% Gxx_ss=gxx(:,index_xx_ss);
% Hx_z=hx(1:nz,1:nz);
% Hxx_zz=hxx(1:nz,index_xx_zz);
% Hxx_zs=hxx(1:nz,index_xx_zs);
% Hxx_ss=hxx(1:nz,index_xx_ss);
%% define discrete state dependent intercepts in ME and SE
% Gxx_zsa=zeros(ny,nz,ncs);
% Hxx_zsa=zeros(nz,nz,ncs);
% for ics=1:ncs
%     icum0=0;
%     for is=1:ns
%         icum1=icum0+nz;        
%         Gxx_zsa(:,:,ics)=Gxx_zsa(:,:,ics)+Gxx_zs(:,icum0+1:icum1)*s_vm(is,ics);
%         Hxx_zsa(:,:,ics)=Hxx_zsa(:,:,ics)+Hxx_zs(:,icum0+1:icum1)*s_vm(is,ics);
%         icum0=icum1;
%     end
% end
%% state equation matrices
a_i=hss(1:nz,:)/2; % this is the part that depends only on discrete states
% this if loop added 04-27-2020
if length(size(hx))==3
    A1_i=hx;
else
    A1_i=repmat(hx,[1 1 ncs]);
end
A2=hxx(:,nre(nz))/2;
B_i=etastd(1:nz,1:nz2,:);
%% measurement equation matrices
c_j=gss/2;
% this if loop added 04-27-2020
if length(size(gx))==3
    C1_j=gx;
else
    C1_j=repmat(gx,[1 1 ncs]);
end
C2=gxx(:,nre(nz))/2;
D=diag(deltav(end-ny+1:end)); 