function [lnconst] = DSGE_prior_lnconst1(R,prior_type_ndxv)
% PURPOSE: computes log prior normalising costant in a DSGE model
% -----------------------------------------------------
% USAGE: 
% where: 
%        R  = (npar x 2) matrix with prior hyperpar(each row a diff par)
%        prior_type_ndxv = (npar x 1) prior type index vector
% -----------------------------------------------------
% NOTE: no dimension checks imposed
%       2010 /03 / 12 (added uniform btw a and b)
%       2022/02/08
% -----------------------------------------------------
% RETURNS: 
%         lnconst = log of normalising constant for prior
% -----------------------------------------------------
% written by:
% Gianni Amisano, Dept of Economics
% University of Brescia
% amisano@eco.unibs.it
npar=size(prior_type_ndxv,1);
lnconst=zeros(npar,1);
% deal with beta distributed parameters
ndx=find(prior_type_ndxv==1);
lnconst(ndx)=-betaln(R(ndx,1),R(ndx,2));
% Gamma distributed parameters (shifted or not)
ndx=find(prior_type_ndxv==2 | prior_type_ndxv==3);
lnconst(ndx)=(R(ndx,2)/2).*log(R(ndx,1)/2)-gammaln(R(ndx,2)/2);
% Gaussian distributed parameters (here no transformation is involved)
ndx=find(prior_type_ndxv==4);
lnconst(ndx)=-log(2*pi)/2-log(R(ndx,2));
% Inverse Gamma distributed parameters 
ndx=find(prior_type_ndxv==5);
lnconst(ndx)=log(2)+(R(ndx,2)/2).*log((R(ndx,1).^2).*R(ndx,2)/2)-gammaln(R(ndx,2)/2);
% U(a,b) distributed parameters
ndx=find(prior_type_ndxv==6);
lnconst(ndx)=-log(abs(R(ndx,2)-R(ndx,1)));



