function [deltav]=DSGE_priordraw_2(R,prior_type_ndxv,nmv)
% PURPOSE: performs draw from prior  in DSGE model 
% -----------------------------------------------------
% USAGE: 
% where: 
% add documentation
% -----------------------------------------------------
% NOTE: no dimension checks imposed , 2007_08_14
% differs from AT1_priordraw because it allows for Inverse Gamma
% (index = 5)
% -----------------------------------------------------
% RETURNS: 
%         deltav = (npar x 1) parameter vector
% -----------------------------------------------------
% written by:
% Gianni Amisano, Dept of Economics
% University of Brescia
% amisano@eco.unibs.it
% 202 02 07 added prior type dirichlet (type 7)
npar=size(R,1);
deltav=zeros(npar,1);
% Beta distributed parameters
ndx=find(prior_type_ndxv==1);
if numel(ndx)>0
    deltav(ndx)= betarnd(R(ndx,1),R(ndx,2));
end
% Gamma distributed parameters
ndx=find(prior_type_ndxv==2);
if numel(ndx)>0
    deltav(ndx)=chi2rnd(R(ndx,2))./R(ndx,1);
end
% shifted Gamma distributed parameters
ndx=find(prior_type_ndxv==3);
if numel(ndx)>0
    deltav(ndx)=1+chi2rnd(R(ndx,2))./R(ndx,1);
end
% Gaussian distributed parameters
ndx=find(prior_type_ndxv==4);
if numel(ndx)>0
    deltav(ndx)=R(ndx,1)+(R(ndx,2).*randn(size(ndx,1),1));
end
% Inverse Gamma distributed parameters
ndx=find(prior_type_ndxv==5);
if numel(ndx)>0
    deltav(ndx)=R(ndx,1)./sqrt(chi2rnd(R(ndx,2))./R(ndx,2));
end
% U(a,b)
ndx=find(prior_type_ndxv==6);
nndx=numel(ndx);
if nndx>0
    av=min(R(ndx,:),[],2);
    bv=max(R(ndx,:),[],2);
    deltav(ndx)=av+(bv-av).*rand(nndx,1);
end
% dirichlet 
ndx=find(prior_type_ndxv==7);
if numel(ndx)
    nc=numel(nmv);
    icum=0;
    for ic=1:nc
        nmi=nmv(ic);
        for im=1:nmi
            % construct the vector of hyperparameters (rem that on first
            % col we read hyperpar on prob of staying on state and on second column we read the
            % hyperpar on probs to leave the state
            rv=R(ndx(icum+1),2)*ones(nmi,1);
            rv(im)=R(ndx(icum+1),1);
            % draw
            deltav(ndx((icum+1):(icum+nmi-1)))=draw_dirichlet(1,nmi,rv,0);
            icum=icum+nmi-1;
        end
    end
end    
    
    
end


