function [retcode,lnpy] = GaussianPdfMatrix1_(Y,M,S)
% computes log pdf (N x 1) for a n x N matrix of 
% N observations on on n-variate random variable
% whose observations are contained in (n x N) matrix Y
% with means given by columns of M (n x N) and 
% common covariance given by (n x n) matrix S
% inputs
%       Y = (n x N) matrix
%       X=  (n x N) means
%       S=  (n x n) covariance matrix
% output:
%       lnpy = (N x 1) vector, whose sum is log pdf of entire sample
% by Gianni Amisano : 2007_12_18
[n,N]=size(Y);
if isnan(rcond(S))
    retcode=0;
    lnpy=[];
else
    [Pp,pscalar]=chol(S);
    if pscalar>0
        retcode=0;
        lnpy=[];
        %pscalar
        %S
        %eig(S)
    else
        retcode=1; 
        Z=(Pp')\(Y-M);
        lnpy=-((n/2)*log(2*pi)+sum(log((diag(Pp)))))*ones(N,1);
        lnpy=lnpy-.5*(sum(Z.^2,1))';
    end
end