function ecdv= empirical_cum_dens(xm,xv,joint)
% function to compute empirical cumulated densities from sample of m variables
% inputs
%       xm    =  (N x m) sample values (each column is a different dimension)
%       xv    =  (m x 1) observed vector
%       joint =  scalar indicator, if not zero then quantile from joint
%                distribution is computed 
% output
%      ecdv   =  (m x 1) vector with empirical marginal cumulative densities
%                 (joint ==0)
%             =  scalar with joint cdf, if joint ne 0
% by Gianni Amisano, amisano@eco.unibs.it, 2013/03/04
[N,m]=size(xm);
if numel(xv)>m
    xv=xv(1:m);
elseif numel(xv)<xv
    xv=mean(xv)*ones(m,1);
end
if joint
    ecdv=mean(sum(xm<repmat(xv',N,1),2)==m)';

else
    ecdv=mean(xm<repmat(xv',N,1))';
end