clear; clc; close all;
%% section 0: general inputs
model_name='RATTS_2020';
Nh=1;
M00=1e+3;
Nh_=40; % horizon for IRF computations
qv0=[.025 .5 .975]';
%% section 1 load results and perform various computations
load(['../../' model_name '_results_postproc']);
% 1.1 compute one step ahead first and second moments
% this computation is done by extrapolating filtered variables
% caution: it is slow
%% Section 1: some inputs
nh=20; % reported max irf horizon 
qv=qv0;
M0=M00;
Mthin=M/M0;
[nz,nz2,ncs]=size(B_i);
ns_=log(ncs)/log(2);
% ndx_label_permute=DSGE_MS_ndx_label_permute_(np,nss); % bug corrected 2013/10/07
% 
%% Section 2: pull GIRFs and make graphs
%% make graphs
load irfaa80;
str_shocks=[];
str_shocks{1}='xi';
str_shocks{2}='z';
str_shocks{3}='pol';
str_shocks{4}='mu';
str_shocks{5}='G';
%
M=size(irfaa,4);
irfaa=irfaa*400;
irfaa(2:3,:,:,:)=cumsum(irfaa(2:3,:,:,:)); %cumulate responses for C and Y

irfaa_EH=zeros(1+3*length(n4),nz2,Nh_+1,M); % store responses to EH long rates and to the "premium" part

irfaa_EH(1:3,:,:,:)=irfaa(4:6,:,1:Nh_+1,:); % responses of I1, I12, I40

for ih=1:Nh_+1
    irfa=mean(irfaa(5,:,ih+(0:11),:),3);
    irfaa_EH(4,:,ih,:)=mean(irfaa(5,:,ih+(0:11),:),3);
    irfaa_EH(5,:,ih,:)=mean(irfaa(5,:,ih+(0:39),:),3);
end
% make impulse responses of "premia", i.e. spot long rates - corresponding EH rates
irfaa_EH(6,:,:,:)=irfaa_EH(2,:,:,:)-irfaa_EH(4,:,:,:); % 3 YEAR
irfaa_EH(7,:,:,:)=irfaa_EH(3,:,:,:)-irfaa_EH(5,:,:,:); % 10 YEAR
%%
% so now we have 7 IRF with the following order
% 1) I01
% 2) I12
% 3) I40
% 4) I12_EH
% 5) I40_EH
% 6) I12-I12_EH
% 7) I40-I40_EH
% define the string for the series 
str_var_irf=[];

str_var_irf{1}='I01';
str_var_irf{2}='I12';
str_var_irf{3}='I40';
str_var_irf{4}='I12EH';
str_var_irf{5}='I40EH';
str_var_irf{6}='I12 minus I12EH';
str_var_irf{7}='I40 minus I40EH';
% back out the relevant posterior moments
irfaa_=mean_std_quantiles(irfaa_EH,qv);
Nh=nh;
% 
% str_var=str_var;
% str_var{2}='C';
% str_var{3}='Y';
% 
for iz=1:nz2
    ymin=min(min(squeeze(irfaa_(:,iz,1:Nh+1,3))));
    ymax=max(max(squeeze(irfaa_(:,iz,1:Nh+1,5))));
    figure(iz);
    for in=1:7
        subplot(3,3,in);
        yyymat=squeeze(irfaa_(in,iz,1:Nh+1,[1 3 4 5]));
        plot((0:Nh)',yyymat(:,1),'LineWidth',3,'Color','red');
        hold on;
        plot((0:Nh)',yyymat(:,3),'LineWidth',1,'LineStyle','--','Color','blue');
        hh=fill([(0:Nh) fliplr(0:Nh)],[yyymat(:,4)' fliplr(yyymat(:,2)')],'b');
        alpha(hh,.05);
        hold off;
        ylabel(str_var_irf(in),'FontSize',15);
        xlabel('horizon','FontSize',12);
        xlim([0 Nh]);
        ylim([ymin ymax]);
        if in==1
            title(strjoin(['response to shock ' str_shocks(iz)]),'FontSize',12);
        end
    end
    figure_filename=['figure_GIRF_EH' str_shocks{iz}];
    saveas(iz,figure_filename,'fig');
    print(iz,'-dpsc2',figure_filename);
    print(iz,'-dpdf',figure_filename);
end
close all;
% here responses of observables variables only
%
irfaa_=mean_std_quantiles(irfaa,qv);

str_var_irf=[];
str_var_irf{1}='\pi';
str_var_irf{2}='c';
str_var_irf{3}='y';
str_var_irf{4}='R_{1}';
str_var_irf{5}='R_{12}';
str_var_irf{6}='R_{40}';

for iz=1:nz2
    ymin=min(min(squeeze(irfaa_(:,iz,1:Nh+1,3))));
    ymax=max(max(squeeze(irfaa_(:,iz,1:Nh+1,5))));
    figure(iz);
    for in=1:6
        subplot(3,2,in);
        yyymat=squeeze(irfaa_(in,iz,1:Nh+1,[1 3 4 5]));
        plot((0:Nh)',yyymat(:,1),'LineWidth',3,'Color','red');
        hold on;
        plot((0:Nh)',yyymat(:,3),'LineWidth',1,'LineStyle','--','Color','blue');
        hh=fill([(0:Nh) fliplr(0:Nh)],[yyymat(:,4)' fliplr(yyymat(:,2)')],'b');
        alpha(hh,.05);
        hold off;
        ylabel(str_var_irf(in),'FontSize',20);
        xlabel('horizon','FontSize',20);
        xlim([0 Nh]);
        ylim([ymin ymax]);
        set(gca,'fontsize',20);
        
        if in==1
            title(strjoin(['Response to shock ' str_shocks(iz)]),'FontSize',20);
        end
    end
    figure_filename=['figure_GIRF_' str_shocks{iz}];
    saveas(iz,figure_filename,'fig');
    print(iz,'-dpsc2',figure_filename);
    print(iz,'-dpdf',figure_filename);
end
% 
% 

close all;



