% Program to compute equity price using RATTS model
% with regime switching shocks

% Version with: [1] three switching states: inflation target and conditional
%                   variance of policy and stationary technology shocks
%               [2] non-expected utility
%               [3] stationary plus INTEGRATED TECH SHOCKS

% Gianni Amisano & Oreste Tristani
% Date: 20 March 2021

% ----------------------------------------------
% Mar 2021: INCLUDES THE REAL EQUITY PRICE INDEX 
%           & the equity premium
%           & the NATURAL RATE OF INTEREST
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y xi_t z_t pol_t mu_t G_t st_pist_t];
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t ERPn_t Rbar_t]; 
%      1    2   3   4   5   6     7     8      9       10    11    12       13  14  15  16  17    18  19     20                                       
% ----------------------------------------------
clear; clc;%
%addpath(genpath('/Users/giovanniamisano/Documents/gianni/research/matlab_code'));
addpath(genpath('/cmc/scratch/m1gga00/matlab_code'));
load ../../RATTS_2020_results;
param=param(:,1:np);
save param.mat param;
clear all;
%%
M=1e+3;
order=2;
% ====================================================
% Parameters
load param;
load ../forecasts/Yo_mv;
Yo_mv=Yo;
load ../forecasts/Yo;
%deltav=delta0;
M_=size(param,1);
nthin=M_/M;
n1 = 10 ;
% Number of non-predetermined variables
n2 = 20;
% x = [lag_I lag_Pi lag_C lag_Y xi_t z_t pol_t mu_t G_t st_pist_t];
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t ERPn_t Rbar_t]; 
%      1    2   3   4   5   6     7     8      9       10    11    12       13  14  15  16  17    18  19     20                                       
% Position of observable in non-predetermined vector
% pie,deltaC,deltaY,r,PiEq_t,ERP_t,ERPbar_t,C_n,R_n
% NOTE: it does not take into account the yield
n3 = [1    8, 9, 5,10,    11,   12,      13, 15, 18, 19,    20];% 
%     Pi_t dc dy r PiEq_t ERP_t ERPbar_t C_n R_n R_t ERPn_t Rbar_t 
% Maturities used in estimation (in quarters)
n4 = [12,40];   % 3 and 10 years
% ====================================================
% data
% ====================================================
nlb=length(n4);
nav=length(n3)-(size(Yo_mv,2)-nlb); % number of additional variables being backed out (excluding yields)
nar=2*nlb; % number of added yields: real and EH nominal, one for each maturity
T=size(Yo_mv,1);
datesv=1966+((0:(T-1)/4))';
Yo_mv_=[Yo_mv(:,1:4) NaN*ones(T,nav) Yo_mv(:,5:6) NaN*ones(T,nar)];
ya=zeros(T,length(n3)+3*length(n4),M);
za=zeros(T,n1,M);
deltam=zeros(M,size(param,2)+nav+nar);
nretcode0=0;
for im=1:M
    deltav=param((im-1)*nthin+1,:)';
    deltav=[deltav(1:38); zeros(nav,1); deltav(39:40); zeros(nar,1)];
    deltam(im,:)=deltav';
    [retcode_ ,y_ ,gx_ ,hx_ ,gxx_ ,hxx_ ,gss_ ,hss_ ,eta_ ,etastd_ ] = RATTS_2021c2_modsol_Klein_( order,deltav,n1,n2,n3,n4);
    % construct matrices and perform filtering and smoothing
    [a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,tpa,~,ctp,pv,s_vm,emv] = ... 
        DSGE_MS_state_space_matrices_simple(gx_,hx_,gxx_,hxx_,gss_,hss_,eta_,etastd_,deltav);
     [retcode,lnlv,pyta_ij,Za,Qa,pm,P01a,Z10a,Q10aa,p1m,sm,simstate,muy_ij_a,Sigmay_ij_a,y10m,zm] = ...
        DSGE_MS_Kim_Filter_pp_mv(Yo_mv_,a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,ctp,pv,1,1,1);
    if retcode==0
        ya(:,:,im)=ya(:,:,im-1);
        za(:,:,im)=za(:,:,im-1);
        deltam(im,:)=deltam(im-1,:);
        nretcode0=nretcode0+1;
    else
    for it=1:T
        s1=simstate(it);
        zv=zm(it,:)';
        yv=c_j(:,s1)+C1_j(:,:,s1)*zv+C2*quadc1(zv);
        ya(it,:,im)=yv';
        za(it,:,im)=zv';
    end
    end
    clc;
    100*im/M
end
%%
save equity_stuff ya za deltam datesv Yo_mv;
