% Program to compute equity price using RATTS model
% with regime switching shocks

% Version with: [1] three switching states: inflation target and conditional
%                   variance of policy and stationary technology shocks
%               [2] non-expected utility
%               [3] stationary plus INTEGRATED TECH SHOCKS

% Gianni Amisano & Oreste Tristani
% Date: 20 March 2021

% ----------------------------------------------
% Mar 2021: INCLUDES THE REAL EQUITY PRICE INDEX 
%           & the equity premium
%           & the NATURAL RATE OF INTEREST
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y xi_t z_t pol_t mu_t G_t st_pist_t];
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t Rbar_t ERPn_t]; 
% ----------------------------------------------
clear; clc;%
%addpath(genpath('/Users/giovanniamisano/Documents/gianni/research/matlab_code'));
addpath(genpath('/cmc/scratch/m1gga00/matlab_code'));
load ../../RATTS_2020_results;
param=param(:,1:np);
save param.mat param;
clear all;
%%
M=1e+0;
order=2;
nh=40;
N=1e+4;
ind_mean=1;
% ====================================================
% Parameters
load param;
load ../forecasts/Yo_mv;
Yo_mv=Yo;
load ../forecasts/Yo;
%deltav=delta0;
M_=size(param,1);
if ind_mean
    M=1;
end
nthin=M_/M;
n1 = 10 ;
% Number of non-predetermined variables
n2 = 20;
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y lag_Cn xi_t z_t pol_t mu_t G_t st_pist_t];
%      1     2      3     4     5      6    7   8     9    10
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t ERPn_t Rbar_t]; 
%      1    2   3   4   5   6     7     8      9       10    11    12       13  14  15  16  17    18  19     20                                       
% NOTE: it does not take into account the yield
n3 = [1,   8, 9, 5,10,    11,   12,      13, 15, 18, 19,    20];% 
%     Pi_t dc dy r PiEq_t ERP_t ERPbar_t C_n R_n R_t ERPn_t Rbar_t 
%     1    2  3  4 5      6     7        8   9   10  11     12   
% Maturities used in estimation (in quarters)
n4 = [12,40];   % 3 and 10 years
% ====================================================
% data
% ====================================================
nlb=length(n4);
nav=length(n3)-(size(Yo_mv,2)-nlb); % number of additional variables being backed out (excluding yields)
nar=2*nlb; % number of added yields: real and EH nominal, one for each maturity
T=size(Yo_mv,1);
datesv=1966+((0:(T-1)/4))';
Yo_mv_=[Yo_mv(:,1:4) NaN*ones(T,nav) Yo_mv(:,5:6) NaN*ones(T,nar)];
nyall=size(Yo_mv_,2);
ya=zeros(T,length(n3)+3*length(n4),M);
za=zeros(T,n1,M);
deltam=zeros(M,size(param,2)+nav+nar);
nz2=5;
nss=3;
ncs=2^nss;
nz1=n1-nz2; % predermined exogs
ndxm=npermutek([1;0],nss);
ndx_mg_states=zeros(ncs/2,nss);
for is=1:nss
    ndx_mg_states(:,is)=find(ndxm(:,is)==1);
end
ndx_me=[1 2 3 4 13 14];
nme=length(ndx_me);
nsources=1+length(ndx_me)+nz2+nss;
fevdaa=zeros(n1+nyall,nh,nsources,M);
deltav=mean(param)';
for im=1:M
    if ind_mean==0
        deltav=param((im-1)*nthin+1,:)';
    end
       % now find solution of model 2021
    %
    deltav=[deltav(1:38); zeros(nav,1); deltav(39:40); zeros(nar,1)];
    deltam(im,:)=deltav';
    % linear solution
    [retcode_ ,y__ ,gx_ ,hx_ ,gxx_ ,hxx_ ,gss_ ,hss_ ,eta_ ,etastd_ ] = RATTS_2021c2_modsol_Klein_( 1,deltav,n1,n2,n3,n4);
    [a_i_,A1_i_,A2_,B_i_,c_j_,C1_j_,C2_,D_,Sigma_zz_,tpa_,pm_,ctp_,pv_,s_vm_,emv_] = ...
        DSGE_MS_state_space_matrices_simple(gx_,hx_,gxx_,hxx_,gss_,hss_,eta_,etastd_,deltav);
    [retcode_ ,y_ ,gx ,hx ,gxx ,hxx ,gss ,hss ,eta ,etastd ] = RATTS_2021c2_modsol_Klein_( 2,deltav,n1,n2,n3,n4);
    [a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,tpa,pm,ctp,pv,s_vm,emv] = ...
        DSGE_MS_state_space_matrices_simple(gx,hx,gxx,hxx,gss,hss,eta,etastd,deltav);
    solution_str=v2struct(a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,a_i_,A1_i_);
    [retcode,lnlv,pyta_ij,Za,Qa,pm,P01a,Z10a,Q10aa,p1m,sm,simstate,muy_ij_a,Sigmay_ij_a,y10m,zm] = ...
        DSGE_MS_Kim_Filter_pp_mv(Yo_mv_,a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,ctp,pv,1,1,1);
    dataa=zeros(n1+nyall,nh,nsources,N);
    % next bloc runs N different simulations
    % from base model (all shocks and switches active)
    % and from models where shocks and switches are being shut off one by one
    for in=1:N
        % simulate discrete process
        % free (first column of s1m) and constrained to no switch
        % (remaining columns
        s1m=zeros(nh+1,1+nss);
        pv=sm(end,:)';
        for ih=0:nh
            s0=draw_multinom(pv,1,ncs,1);
            s1m(ih+1,1)=s0;
            pv=ctp(s0,:)';
            s0_=ndxm(s0,:)';
            % this is to freeze state = 1 for each of the discrete
            % processes
            for is=1:nss
                s0__=s0_;
                s0__(is)=1;
                [~,is_]=ismember(s0__',ndxm,'rows');
                s1m(ih+1,1+is)=is_;
            end
        end
        z0=zm(end,:)';
        em=randn(nyall+nz2,nh);
        % baseline simulation with all shocks
        dataa(:,:,end,in)     = DSGE_MS_dynamic_simulation(solution_str, z0,em,s1m(:,1),1);
        solution_str0=solution_str;
        % now kill one at a time continuous shocks
        for iz2=1:nz2
            em0=em;
            em0(iz2,:)=0;
            dataa(:,:,iz2,in)=DSGE_MS_dynamic_simulation(solution_str, z0,em0,s1m(:,1),1);
        end
        icum=nz2;
        % now take off each of the switches
        for is=1:nss
            dataa(:,:,icum+is,in)=DSGE_MS_dynamic_simulation(solution_str, z0,em,s1m(:,1+is),1);
        end
        icum=icum+nss;
        % now kill one by one measurement errors
        %ndx_me=[1 2 3 4 13 14];
        for ime=1:nme
            em0=em;
            em0(nz2+ndx_me(ime),:)=0;
            dataa(:,:,icum+ime,in)=DSGE_MS_dynamic_simulation(solution_str, z0,em0,s1m(:,1),1);
        end
        clc;
        in/N
    end
    vara=var(dataa,0,4);
    fevda=zeros(n1+nyall,nh,nsources);
    % from 1 to nz2, the continuous shocks
    fevda(:,:,1:nz2)=1-squeeze(vara(:,:,1:nz2))./repmat(squeeze(vara(:,:,end)),[1 1 nz2]);
    % now the discrete shocks
    icum=nz2;
    fevda(:,:,icum+1:icum+nss)=1-squeeze(vara(:,:,icum+1:icum+nss))./repmat(squeeze(vara(:,:,end)),[1 1 nss]);
    icum=icum+nss;
    %remember to subtract effect of switching from those of the corresponding
    %continuous shock (zi and pol are respectively shock 2 and shock 3,
    %corresponding to discrete regimes 3 and 2 (odd, but this is what it is)
    fevda(:,:,[2 3])=fevda(:,:,[2 3])-fevda(:,:,nz2+[3 2]);
    % now the measurement errors
    fevda(:,:,icum+1:icum+nme)=1-squeeze(vara(:,:,icum+1:icum+nme))./repmat(squeeze(vara(:,:,end)),[1 1 nme]);
    clc;
    disp(['computing FEVD, ' num2str(100*im/M) ' percent completed']);
    fevdaa(:,:,:,im)=fevda;
end



save equity_fevda fevdaa

