% Program to compute equity price using RATTS model
% with regime switching shocks

% Version with: [1] three switching states: inflation target and conditional
%                   variance of policy and stationary technology shocks
%               [2] non-expected utility
%               [3] stationary plus INTEGRATED TECH SHOCKS

% Gianni Amisano & Oreste Tristani
% Date: 20 March 2021

% ----------------------------------------------
% Mar 2021: INCLUDES THE REAL EQUITY PRICE INDEX 
%           & the equity premium
%           & the NATURAL RATE OF INTEREST
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y xi_t z_t pol_t mu_t G_t st_pist_t];
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t Rbar_t ERPn_t]; 
% ----------------------------------------------
clear; clc;%
%addpath(genpath('/Users/giovanniamisano/Documents/gianni/research/matlab_code'));
addpath(genpath('/cmc/scratch/m1gga00/matlab_code'));
load ../../RATTS_2020_results;
param=param(:,1:np);
save param.mat param;
clear all;
%%
M=5e+3;
order=2;
nh=80;
ndx_mean=0;
ndx_prune=1;
initial_state=[1 1 1];
ndx_other_states=1; % if other states are allowed to switch in the simulation
ndx_permanent=0;
% ====================================================
% Parameters
load param;
load ../forecasts/Yo_mv;
Yo_mv=Yo;
load ../forecasts/Yo;
%deltav=delta0;
M_=size(param,1);
if ndx_mean
    M=1;
end
nthin=M_/M;
n1 = 10 ;
% Number of non-predetermined variables
n2 = 20;
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y lag_Cn xi_t z_t pol_t mu_t G_t st_pist_t];
%      1     2      3     4     5      6    7   8     9    10
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t ERPn_t Rbar_t]; 
%      1    2   3   4   5   6     7     8      9       10    11    12       13  14  15  16  17    18  19     20                                       
n3 = [1,8,9,5,10,11,12,13,15,18,19,20];% 
%     Pi_t dc dy r PiEq_t ERP_t ERPbar_t C_n R_n R_t ERPn_t Rbar_t 
% Maturities used in estimation (in quarters)
n4 = [12,40];   % 3 and 10 years
% ====================================================
% data
% ====================================================
nlb=length(n4);
nav=length(n3)-(size(Yo_mv,2)-nlb); % number of additional variables being backed out (excluding yields)
nar=2*nlb; % number of added yields: real and EH nominal, one for each maturity
T=size(Yo_mv,1);
datesv=1966+((0:(T-1))/4)';
Yo_mv_=[Yo_mv(:,1:4) NaN*ones(T,nav) Yo_mv(:,5:6) NaN*ones(T,nar)];
nyall=size(Yo_mv_,2);
deltam=zeros(M,size(param,2)+nav+nar);
%
nz2=5;
nss=3;
ncs=2^nss;
%
ndxm=npermutek([1;0],nss);
ndx_mg_states=zeros(ncs/2,nss);
for is=1:nss
    ndx_mg_states(:,is)=find(ndxm(:,is)==1);
end
ndx_var_2020=[(1:(nz2+4))'; nz2+length(n3)+(1:2)'];
irfa=zeros(nyall+n1,nh+1,nss,2,M);
irfa_=zeros(15,nh+1,nss,2,M);
%%
for im=1:M
    if ndx_mean
        deltav=mean(param)';
    else
        deltav=param((im-1)*nthin+1,:)';
    end
    % find solution of model 2020
    deltav0=deltav;
    % linear solution
    [retcode_ ,y_20_1 ,gx_20_1 ,hx_20_1 ,gxx_20_1 ,hxx_20_1 ,gss_20_1 ,hss_20_1 ,eta_20_1 ,etastd_20_1 ] = RATTS_2020_modsol_Klein_( 1,deltav,n1-1,9,[1 8 9 5],n4);
    [a_i_20_1,A1_i_20_1,A2_20_1,B_i_20_1,c_j_20_1,C1_j_20_1,C2_20_1,D_20_1,Sigma_zz_20_1,tpa_20_1,pm_20_1,ctp_20_1,pv_20_1,s_vm_20_1,emv_20_1] = ...
        DSGE_MS_state_space_matrices_simple(gx_20_1,hx_20_1,gxx_20_1,hxx_20_1,gss_20_1,hss_20_1,eta_20_1,etastd_20_1,deltav);
    solution_str_20_1=v2struct(a_i_20_1,A1_i_20_1,A2_20_1,B_i_20_1,c_j_20_1,C1_j_20_1,C2_20_1,D_20_1,a_i_20_1,A1_i_20_1);
    % quadratic solution
    [retcode_ ,y_20_2 ,gx_20_2 ,hx_20_2 ,gxx_20_2 ,hxx_20_2 ,gss_20_2 ,hss_20_2 ,eta_20_2 ,etastd_20_2 ] = RATTS_2020_modsol_Klein_( 2,deltav,n1-1,9,[1 8 9 5],n4);
    [a_i_20_2,A1_i_20_2,A2_20_2,B_i_20_2,c_j_20_2,C1_j_20_2,C2_20_2,D_20_2,Sigma_zz_20_2,tpa_20_2,pm_20_2,ctp_20_2,pv_20_2,s_vm_20_2,emv_20_2] = ...
        DSGE_MS_state_space_matrices_simple(gx_20_2,hx_20_2,gxx_20_2,hxx_20_2,gss_20_2,hss_20_2,eta_20_2,etastd_20_2,deltav);
    solution_str_20_2=v2struct(a_i_20_2,A1_i_20_2,A2_20_2,B_i_20_2,c_j_20_2,C1_j_20_2,C2_20_2,D_20_2,a_i_20_1,A1_i_20_1);
    %
    % now find solution of model 2021
    %
    deltav=[deltav(1:38); zeros(nav,1); deltav(39:40); zeros(nar,1)];
    deltam(im,:)=deltav';
    % linear solution
    [retcode_ ,y_21_1 ,gx_21_1 ,hx_21_1 ,gxx_21_1 ,hxx_21_1 ,gss_21_1 ,hss_21_1 ,eta_21_1 ,etastd_21_1 ] = RATTS_2021c2_modsol_Klein_( 1,deltav,n1,n2,n3,n4);
    [a_i_21_1,A1_i_21_1,A2_21_1,B_i_21_1,c_j_21_1,C1_j_21_1,C2_21_1,D_21_1,Sigma_zz_21_1,tpa_21_1,pm_21_1,ctp_21_1,pv_21_1,s_vm_21_1,emv_21_1] = ...
        DSGE_MS_state_space_matrices_simple(gx_21_1,hx_21_1,gxx_21_1,hxx_21_1,gss_21_1,hss_21_1,eta_21_1,etastd_21_1,deltav);
    solution_str_21_1=v2struct(a_i_21_1,A1_i_21_1,A2_21_1,B_i_21_1,c_j_21_1,C1_j_21_1,C2_21_1,D_21_1,a_i_21_1,A1_i_21_1);
    % quadratic solution
    [retcode_ ,y_21_2 ,gx_21_2 ,hx_21_2 ,gxx_21_2 ,hxx_21_2 ,gss_21_2 ,hss_21_2 ,eta_21_2 ,etastd_21_2 ] = RATTS_2021c2_modsol_Klein_( 2,deltav,n1,n2,n3,n4);
    [a_i_21_2,A1_i_21_2,A2_21_2,B_i_21_2,c_j_21_2,C1_j_21_2,C2_21_2,D_21_2,Sigma_zz_21_2,tpa_21_2,pm_21_2,ctp_21_2,pv_21_2,s_vm_21_2,emv_21_2] = ...
        DSGE_MS_state_space_matrices_simple(gx_21_2,hx_21_2,gxx_21_2,hxx_21_2,gss_21_2,hss_21_2,eta_21_2,etastd_21_2,deltav);
    solution_str_21_2=v2struct(a_i_21_2,A1_i_21_2,A2_21_2,B_i_21_2,c_j_21_2,C1_j_21_2,C2_21_2,D_21_2,a_i_21_1,A1_i_21_1);
    pv=pv_21_2; % any will do
    epm=zeros(2,nss);
    for is=1:nss
        epm(1,is)=sum(pv(ndxm(:,is)==1));
        epm(2,is)=sum(pv(ndxm(:,is)==0));
    end
    clc;
    z0=zeros(n1,1);
    z0(2)=epm(2,1)*(log(deltav(7)/deltav(8)));
    z0_=z0([1 2 3 4 6 7 8 9 10]);
    ema=randn(nyall+nz2,nh+1,2);
    ema(:,:,2)=0;
    % produce scenarios for discrete variables
    ctp=ctp_21_2;
    tpa=tpa_21_2;
    [sm0,sa1]  = DSGE_MS_GIRF_discrete_states(1+nh,initial_state,ctp,tpa,ndx_other_states,ndx_permanent,ndxm);
    for is=1:nss
        sv1=squeeze(sa1(:,nss+1,is,1));
        sv2=squeeze(sa1(:,nss+1,is,2));
        for imode=1:2
            em=squeeze(ema(:,:,imode));
            datam1     = DSGE_MS_dynamic_simulation(solution_str_21_2, z0,em,sv1,ndx_prune);
            datam2     = DSGE_MS_dynamic_simulation(solution_str_21_2, z0,em,sv2,ndx_prune);
            irfa(:,:,is,imode,im)=(datam2-datam1);
            %
            em=squeeze(ema(ndx_var_2020,:,imode));
            datam1     = DSGE_MS_dynamic_simulation(solution_str_20_2, z0_,em,sv1,ndx_prune);
            datam2     = DSGE_MS_dynamic_simulation(solution_str_20_2, z0_,em,sv2,ndx_prune);
            irfa_(:,:,is,imode,im)=(datam2-datam1);

            

        end
    end
       
    clc;
    disp(['computing IRFs, ' num2str(100*im/M) ' percent completed']);
end
%
filename0=['MS_IRFs_ini' int2str(initial_state)];
filename0=strrep(filename0,' ','');
if ndx_other_states
filename0=[filename0 '_swiothers'];
else
    filename0=[filename0 '_fixothers'];
end
if ndx_permanent
filename0=[filename0 '_perm'];
else
    filename0=[filename0 '_temp'];
end
%%
% ordering of variables for IRF computations
% x' y'
%
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y lag_Cn xi_t z_t pol_t mu_t G_t st_pist_t];
%      1     2      3     4     5      6    7   8     9    10
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t ERPn_t Rbar_t]; 
%      1    2   3   4   5   6     7     8      9       10    11    12       13  14  15  16  17    18  19     20                                       
%n3 = [1,8,9,5,10,11,12,13,15,18,19,20];% 
%     Pi_t dc dy r PiEq_t ERP_t ERPbar_t C_n R_n R_t Rbar_t ERPn_t
% hence ordering for IRF computations is 
% x' y'  =  lag_I lag_Pi lag_C lag_Y lag_Cn xi_t z_t pol_t mu_t G_t;
%           1     2      3     4     5      6    7   8     9    10  
%           Pi_t  dc     dy    r     PiEq_t ERP_t ERPbar_t C_n R_n R_t ERPn_t Rbar_t      
%           11    12     13    14    15     16    17       18  19  20  21     22
%           R12 R40 RR12 RR40 EHR12 EHR40
%           23  24  25   26   27    28
%
close all;
%load equity_girf
% added two rows in irfa to report IRFs of nominal, not detrended PIEq
% 1) PiEq_nom in nominal and not detrended form (PIEq_t + cumsum(Pi+xi))
% notice that we do not need to add mean growth rate of xi which would
% cancel out in computations
% 2) same thing but in real terms, i.e. without adding cumsum of inflation
% (variable 11)
irfa=cat(1,irfa,sum(cumsum(irfa([6 11],:,:,:,:),2),1)+irfa(15,:,:,:,:));
irfa=cat(1,irfa,cumsum(irfa(6,:,:,:,:),2)+irfa(15,:,:,:,:));
%%
str_var={'lag_I','lag_Pi','lag_C','lag_Y','lag_Cn','xi_t','z_t','pol_t','mu_t','G_t', ...
    'Pi_t','dc','dy','r','PiEq_t','ERP_t','ERPbar_t ','C_n','R_n','R_t','ERPn_t', 'Rbar_t'...
    'R12','R40','RR12','RR40','EHR12','EHR40','PiEqbar_nominal','PiEqbar_real'};
str_var_=strrep(str_var,'_','\_');
str_states_={'pistar','pol','zi'};
qv=[.05 .1 .25 .5 .75 .9 .95]';
xv=0:nh;
irfa_rep =mean_std_quantiles(irfa, qv);
irfa_rep_=mean_std_quantiles(irfa_,qv);

nh_=40;
xv_=xv(1:nh_+1);

% first panel is pi, dc, dy, r; second panel is r12 r40 rr12 rr40; 
% third panel is EH12 EH40 ERPn Rbar
% fourth panel Rn PiEq PiEqbar
ndx_reportm=[11 12 13 14; 23 24 25 26; 27 28 21 22; 19 15 30 29]; 
ndx_reportqv=[1 3 5 7]';

nfig=0;
clc;
ndx_reported_var=zeros(numel(ndx_reportm),2);
for itype=1:2
    icum=0;
    for ifig=1:size(ndx_reportm,1)
        nfig=nfig+1;
        figure(nfig);
        ndx_var_ifig=ndx_reportm(ifig,ndx_reportm(ifig,:)~=0)'; % purge elements of row ifig being equal to zero
        %ndx_var_ifig
        for in=1:length(ndx_var_ifig)
            icum=icum+1;
            in1=ndx_var_ifig(in);
            ndx_reported_var(icum,itype)=in1;
            for is=1:nss
                ym_=400*squeeze(irfa_rep(in1,1:(nh_+1),is,itype,[1; 2+ndx_reportqv]));
                subplot(size(ndx_reportm,2),nss,(in-1)*nss+is);
                plot1=plot(xv_,ym_);
                set(plot1(1),'Color','r','LineWidth',3);
                for iline=2:length(ndx_reportqv)+1 % all lines (not reported the median)
                    set(plot1(iline),'Color','k','LineWidth',2,'LineStyle',':');
                end
                %set(gca,'FontSize',10');
                if in==1
                    title(str_states_(is));
                end
                xlabel(str_var_(in1));
            end
        end
        filename=[filename0 '_type' int2str(itype) '_part_' int2str(ifig)];
        sgtitle(strrep(filename,'_','\_'));
        orient(nfig,'landscape');
        %print(nfig,'-fillpage',filename,'-dpdf');
        print(nfig,'-dpdf',filename);
        saveas(nfig,filename,'fig');
        %title(
    end
end
save(filename0,'filename0','deltam','irfa','irfa_','irfa_rep','irfa_rep_','str_var','qv', ...
    'initial_state','ndx_other_states','ndx_permanent');
