clear; clc; close all;

%% Import data from spreadsheet
% Script for importing data from the following spreadsheet:
%
%    Workbook: /Users/giovanniamisano/Desktop/RATTS_2019/RATTS_2019_newdata/feds200628.xlsx
%    Worksheet: Yields
%
% Auto-generated by MATLAB on 31-Oct-2019 19:22:22

%% Setup the Import Options
opts = spreadsheetImportOptions("NumVariables", 103);

% Specify sheet and range
opts.Sheet = "Yields";
opts.DataRange = "A11:CY14567";

% Specify column names and types
opts.VariableNames = ["VarName1", "day", "month", "year", "SVENY01", "SVENY02", "SVENY03", "SVENY04", "SVENY05", "SVENY06", "SVENY07", "SVENY08", "SVENY09", "SVENY10", "SVENY11", "SVENY12", "SVENY13", "SVENY14", "SVENY15", "SVENY16", "SVENY17", "SVENY18", "SVENY19", "SVENY20", "SVENY21", "SVENY22", "SVENY23", "SVENY24", "SVENY25", "SVENY26", "SVENY27", "SVENY28", "SVENY29", "SVENY30", "SVENPY01", "SVENPY02", "SVENPY03", "SVENPY04", "SVENPY05", "SVENPY06", "SVENPY07", "SVENPY08", "SVENPY09", "SVENPY10", "SVENPY11", "SVENPY12", "SVENPY13", "SVENPY14", "SVENPY15", "SVENPY16", "SVENPY17", "SVENPY18", "SVENPY19", "SVENPY20", "SVENPY21", "SVENPY22", "SVENPY23", "SVENPY24", "SVENPY25", "SVENPY26", "SVENPY27", "SVENPY28", "SVENPY29", "SVENPY30", "SVENF01", "SVENF02", "SVENF03", "SVENF04", "SVENF05", "SVENF06", "SVENF07", "SVENF08", "SVENF09", "SVENF10", "SVENF11", "SVENF12", "SVENF13", "SVENF14", "SVENF15", "SVENF16", "SVENF17", "SVENF18", "SVENF19", "SVENF20", "SVENF21", "SVENF22", "SVENF23", "SVENF24", "SVENF25", "SVENF26", "SVENF27", "SVENF28", "SVENF29", "SVENF30", "SVEN1F01", "SVEN1F04", "SVEN1F09", "BETA0", "BETA1", "BETA2", "BETA3", "TAU1", "TAU2"];
opts.VariableTypes = ["double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double"];

% Import the data
feds200628 = readtable("feds200628.xlsx", opts, "UseExcel", false);

%% Convert to output type
feds200628 = table2array(feds200628);
% Clear temporary variables
VariableNames=opts.VariableNames;
clear  opts
%%

alldata=flipud(feds200628); % sort the data from sample start to end
% get the columns with the NS coefficients
beta0v=alldata(:,98);
beta1v=alldata(:,99);
beta2v=alldata(:,100);
beta3v=alldata(:,101);
tau1v=alldata(:,102);
tau2v=alldata(:,103);
% compute the 10 year rate with the full formula
m=10; % maturity expressed in years
y40q  =  beta0v + beta1v.*(1 - exp(-m./tau1v))./(m./tau1v) ...
        + beta2v.*((1 - exp(-m./tau1v))./(m./tau1v)  - exp(-m./tau1v)) ...
        + beta3v.*((1 - exp(-m./tau2v))./(m./tau2v) - exp(-m./tau2v));        %spot rate
%%
% find index for date when data for SVEN10Y 
T=size(alldata,1);
T0=find(alldata(:,2)==16 & alldata(:,3)==8 & alldata(:,4)==1971);
% compute now t40y_ using simplified formula with beta3=0, tau2=1;
tau2v=ones(T,1);
beta3v=zeros(T,1);
y40q_ =  beta0v + beta1v.*(1 - exp(-m./tau1v))./(m./tau1v) ...
        + beta2v.*((1 - exp(-m./tau1v))./(m./tau1v)  - exp(-m./tau1v)) ...
        + beta3v.*((1 - exp(-m./tau2v))./(m./tau2v) - exp(-m./tau2v));        %spot rate
% extract the 10 year rate from the feds data matrix (14th column)
fedy40q=alldata(:,14);
    
% use the simplified NS until 1971:08:15 to fill NaN in fedy40q and create fedy40q_ 
fedy40q_=fedy40q;
fedy40q_(1:T0-1)=y40q_(1:T0-1);
% define now matrix with dates (day, month,year)
datesm=alldata(:,[2 3 4]);
%%
% now create monthly data using averages of intramonthly data
nyears=datesm(end,3)-datesm(1,3); 
if datesm(1,2)<=3
    nyears=nyears+1; % if the first obs belongs to first quarter then this is the number of full years
end
dseriesv=fedy40q_;
month2q=[1 2 3; 4 5 6; 7 8 9; 10 11 12]; 

initialm=datesm(1,2);
initialy=datesm(1,3);
endy=datesm(end,3);
q0=find(initialm>=[1 4 7 10] & initialm<=[3 6 9 12]);
datesm_=[datesm (1:T)'];


Tq= (datesm(end,3)-datesm(1,3)+1)*4; % complete number
qdatesv=initialy+(0:Tq-1)'/4;
qseriesv=NaN*ones(Tq,1);
icum=q0-1; 
for iy = initialy:endy % pick year
    % extract year
    datesm_y=datesm_(datesm_(:,3)==iy,:);
    for iq=q0:4
        icum=icum+1;
        % define months in the quarter
        monthv=month2q(iq,:)';
        % extract quarter
        datesm_yq=datesm_y(datesm_y(:,2)==monthv(1) | datesm_y(:,2)==monthv(2) |datesm_y(:,2)==monthv(3),:);
        seriesv_=dseriesv(datesm_yq(:,end));
        qseriesv(icum)=mean(seriesv_(~isnan(seriesv_)));
    end
    q0=1; % from the second year on we start on quarter 1
end
qdatam=[qdatesv qseriesv];

% now dump the series into an excel file
str_var_={'obs','qdatesv','y40q'};
T=array2table([(1:length(qdatesv))' qdatesv qseriesv],'VariableNames',str_var_);
writetable(T,'RATTS_2019_y40q.xlsx');
