% Program to compute equity price using RATTS model
% with regime switching shocks

% Version with: [1] three switching states: inflation target and conditional
%                   variance of policy and stationary technology shocks
%               [2] non-expected utility
%               [3] stationary plus INTEGRATED TECH SHOCKS

% Gianni Amisano & Oreste Tristani
% Date: 20 March 2021

% ----------------------------------------------
% Mar 2021: INCLUDES THE REAL EQUITY PRICE INDEX 
%           & the equity premium
%           & the NATURAL RATE OF INTEREST
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y xi_t z_t pol_t mu_t G_t st_pist_t];
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t Rbar_t ERPn_t]; 
% ----------------------------------------------
clear; clc;%
%addpath(genpath('/Users/giovanniamisano/Documents/gianni/research/matlab_code'));
addpath(genpath('/cmc/scratch/m1gga00/matlab_code'));
load ../../../RATTS_2020_asy_prior_2021/vp2/estimation_1966_2009_missing_short_rate/RATTS_2020_results.mat;
param=param(:,1:np);
save param.mat param;
clear all;
%%
M=1e+0;
order=2;
nh=200;
N=1e+4;
ind_mean=1;
% ====================================================
% Parameters
load param;
load ../../../RATTS_2020_asy_prior_2021/vp2/estimation_1966_2009_missing_short_rate/extensive_postprocessing/forecasts/Yo_mv;
Yo_mv=Yo;
load ../../../RATTS_2020_asy_prior_2021/vp2/estimation_1966_2009_missing_short_rate/extensive_postprocessing/forecasts/Yo;
%deltav=delta0;
M_=size(param,1);
if ind_mean
    M=1;
end
nthin=M_/M;
n1 = 10 ;
% Number of non-predetermined variables
n2 = 20;
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y lag_Cn xi_t z_t pol_t mu_t G_t st_pist_t];
%      1     2      3     4     5      6    7   8     9    10
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY PiEq_t ERP_t ERPbar_t C_n Y_n R_n J_n dum_n R_t ERPn_t Rbar_t]; 
%      1    2   3   4   5   6     7     8      9       10    11    12       13  14  15  16  17    18  19     20                                       
% NOTE: it does not take into account the yield
n3 = [1,   8, 9, 5,10,    11,   12,      13, 15, 18, 19,    20];% 
%     Pi_t dc dy r PiEq_t ERP_t ERPbar_t C_n R_n R_t ERPn_t Rbar_t 
%     1    2  3  4 5      6     7        8   9   10  11     12   
% Maturities used in estimation (in quarters)
n4 = [12,40];   % 3 and 10 years
% ====================================================
% data
% ====================================================
nlb=length(n4);
nav=length(n3)-(size(Yo_mv,2)-nlb); % number of additional variables being backed out (excluding yields)
nar=2*nlb; % number of added yields: real and EH nominal, one for each maturity
T=size(Yo_mv,1);
datesv=1966+((0:(T-1)/4))';
Yo_mv_=[Yo_mv(:,1:4) NaN*ones(T,nav) Yo_mv(:,5:6) NaN*ones(T,nar)];
nyall=size(Yo_mv_,2);
ya=zeros(T,length(n3)+3*length(n4),M);
za=zeros(T,n1,M);
deltam=zeros(M,size(param,2)+nav+nar);
nz2=5;
nss=3;
ncs=2^nss;
nz1=n1-nz2; % predermined exogs
ndxm=npermutek([1;0],nss);
ndx_mg_states=zeros(ncs/2,nss);
for is=1:nss
    ndx_mg_states(:,is)=find(ndxm(:,is)==1);
end
ndx_me=[1 2 3 4 13 14];
nme=length(ndx_me);
nsources=length(ndx_me)+nz2+nss;
fevdaa =zeros(n1+nyall+2,nh,nsources,M);
fevdaa_=zeros(n1+nyall+2,nh,nsources,M);

deltav=mean(param)';
for im=1:M
    if ind_mean==0
        deltav=param((im-1)*nthin+1,:)';
    end
       % now find solution of model 2021
    %
    deltav=[deltav(1:38); zeros(nav,1); deltav(39:40); zeros(nar,1)];
    deltam(im,:)=deltav';
    % linear solution
    [retcode_ ,y__ ,gx_ ,hx_ ,gxx_ ,hxx_ ,gss_ ,hss_ ,eta_ ,etastd_ ] = RATTS_2021c2_modsol_Klein_( 1,deltav,n1,n2,n3,n4);
    [a_i_,A1_i_,A2_,B_i_,c_j_,C1_j_,C2_,D_,Sigma_zz_,tpa_,pm_,ctp_,pv_,s_vm_,emv_] = ...
        DSGE_MS_state_space_matrices_simple(gx_,hx_,gxx_,hxx_,gss_,hss_,eta_,etastd_,deltav);
    [retcode_ ,y_ ,gx ,hx ,gxx ,hxx ,gss ,hss ,eta ,etastd ] = RATTS_2021c2_modsol_Klein_( 2,deltav,n1,n2,n3,n4);
    [a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,tpa,pm,ctp,pv,s_vm,emv] = ...
        DSGE_MS_state_space_matrices_simple(gx,hx,gxx,hxx,gss,hss,eta,etastd,deltav);
    solution_str=v2struct(a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,a_i_,A1_i_);
    [retcode,lnlv,pyta_ij,Za,Qa,pm,P01a,Z10a,Q10aa,p1m,sm,simstate,muy_ij_a,Sigmay_ij_a,y10m,zm] = ...
        DSGE_MS_Kim_Filter_pp_mv(Yo_mv_,a_i,A1_i,A2,B_i,c_j,C1_j,C2,D,Sigma_zz,ctp,pv,1,1,1);
    dataa=zeros(n1+nyall+2,nh,nsources+1,N);
    % next bloc runs N different simulations
    % from base model (all shocks and switches active)
    % and from models where shocks and switches are being shut off one by one
    for in=1:N
        % simulate discrete process
        % free (first column of s1m) and constrained to no switch
        % (remaining columns
        s1m=zeros(nh+1,1+nss);
        pv=sm(end,:)';
        for ih=0:nh
            s0=draw_multinom(pv,1,ncs,1);
            s1m(ih+1,1)=s0;
            pv=ctp(s0,:)';
            s0_=ndxm(s0,:)';
            % this is to freeze state = 1 for each of the discrete
            % processes
            for is=1:nss
                s0__=s0_;
                s0__(is)=1;
                [~,is_]=ismember(s0__',ndxm,'rows');
                s1m(ih+1,1+is)=is_;
            end
        end
        z0=zm(end,:)';
        s0=s1m(1,1);
        y0=c_j(:,s0)+C1_j(:,:,s0)*z0+C2*vech_(z0*z0')+D*randn(size(D,2),1); % this is as starting point to compute first differences (useful for computing equity price conditional variances
        em=randn(nyall+nz2,nh);
        % baseline simulation with all shocks
        dataa(1:end-2,:,end,in)     = DSGE_MS_dynamic_simulation(solution_str, z0,em,s1m(:,1),1);
        solution_str0=solution_str;
        % now kill one at a time continuous shocks
        for iz2=1:nz2
            em0=em;
            em0(iz2,:)=0;
            dataa(1:end-2,:,iz2,in)=DSGE_MS_dynamic_simulation(solution_str, z0,em0,s1m(:,1),1);
        end
        icum=nz2;
        % now take off each of the switches
        for is=1:nss
            dataa(1:end-2,:,icum+is,in)=DSGE_MS_dynamic_simulation(solution_str, z0,em,s1m(:,1+is),1);
        end
        icum=icum+nss;
        % now kill one by one measurement errors
        ndx_me=[1 2 3 4 13 14];
        for ime=1:nme
            em0=em;
            em0(nz2+ndx_me(ime),:)=0;
            dataa(1:end-2,:,icum+ime,in)=DSGE_MS_dynamic_simulation(solution_str, z0,em0,s1m(:,1),1);
        end
        clc;
        100*in/N
    end
    % add the final two series ()
    % added two rows in dataa to report
    % 29) PiEq_nom in nominal and not detrended form (PIEq_t + cumsum(Pi+xi))
    %      notice that we do not need to add mean growth rate of xi which would
    %      not affect variance computations
    % 30) same thing but in real terms, i.e. without adding cumsum of inflation
    %     (variable 11)
    dataa(n1+nyall+1,:,:,:,:)=squeeze(sum(dataa([6 11],:,:,:,:),1))+diff([repmat(y0(15),[1 nsources+1 N]) ;squeeze(dataa(15,:,:,:,:))]);
    dataa(n1+nyall+2,:,:,:,:)=squeeze(    dataa( 6    ,:,:,:,:)   )+diff([repmat(y0(15),[1 nsources+1 N]) ;squeeze(dataa(15,:,:,:,:))]);
    vara=var(dataa,0,4);
    fevda=zeros(n1+nyall+2,nh,nsources);
    % from 1 to nz2, the continuous shocks
    fevda(:,:,1:nz2)=max(1-squeeze(vara(:,:,1:nz2))./repmat(squeeze(vara(:,:,end)),[1 1 nz2]),0);
    % now the discrete shocks
    icum=nz2;
    fevda(:,:,icum+1:icum+nss)=max(1-squeeze(vara(:,:,icum+1:icum+nss))./repmat(squeeze(vara(:,:,end)),[1 1 nss]),0);
    icum=icum+nss;
    %remember to subtract effect of switching from those of the corresponding
    %continuous shock (zi and pol are respectively shock 2 and shock 3,
    %corresponding to discrete regimes 3 and 2 (odd, but this is what it is)
    fevda(:,:,[2 3])=fevda(:,:,[2 3])-fevda(:,:,nz2+[3 2]);
    % now the measurement errors
    fevda(:,:,icum+1:icum+nme)=max(1-squeeze(vara(:,:,icum+1:icum+nme))./repmat(squeeze(vara(:,:,end)),[1 1 nme]),0);
    clc;
    %
    % renormalize to sum to 100
    fevda(fevda<0)=0;
    %fevda=100*fevda;
    fevda_=100*fevda./repmat(sum(fevda,3),[1 1 nsources]); % to renormalize to sum to 100
    %
    disp(['computing FEVD, ' num2str(100*im/M) ' percent completed']);
    fevdaa_(:,:,:,im)=fevda_;
    fevdaa( :,:,:,im)=fevda ;
end


save ../output/equity_fevda fevdaa fevda fevdaa_ fevda_ vara% the objects with aa at the end are prepared in case we want to average across draws (here not used)
%% decide what to report and report it
fevda=fevda_;
% compact the measurement errors sources : create repfevda that cumulat
repfevda=zeros(size(fevda,1),size(fevda,2),size(fevda,3)-5);
repfevda(:,:,1:8)=    fevda(:,:,1:8  );
repfevda(:,:,9  )=sum(fevda(:,:,9:end),3);
str_var= {...
    '$I_{t-1}$'      , ...
    '$\pi_{t-1}$'    , ...
    '$\C_{t-1}$'     , ...
    '$\Y_{t-1}$'     , ...
    '$\Cn_{t-1}$'    , ...
    '$\xi_t$'        , ...
    '$z_t$'          , ...
    '$pol_t$'        , ...
    '$\mu_t$'        , ...
    '$G_t$'          , ...
    '$\pi_t$'        , ...
    '$\Delta c$'     , ...
    '$ \Delta y$'    , ...
    '$r$'            , ...
    '$PiEq_t$'       , ...  
    '$ERP_t$'        , ...
    '$ERPbar_t$'     , ...
    '$C_n$'          , ...
    '$R_n$'          , ...
    '$R_t$'          , ...
    '$ERPn_t$'       , ...
    '$Rbar_t$'       , ...
    '$R12$'          , ... 
    '$R40$'          , ...
    '$RR12$'         , ...
    '$RR40$'         , ...
    '$EHR12$'        , ...
    '$EHR40$'        , ...
    '$PiEqbar\_nominal$'       , ...  
    '$PiEqbar\_real$'          , ...
};


str_sou= {...
    '$\xi$'      , ...
    '$z$'      , ...
    '$pol$'      , ...
    '$\mu$'      , ...
    '$G$'      , ...
    '$s_{\pi^*}$'      , ...
    '$s_{pol}$'      , ...
    '$s_z$'      , ...
    '$meas.err.$'        ...
};

ndx_sel_var= [11 12 13 14 22 19 28 24 21 30]';
ndx_sel_hor=[1 12 40 nh];
str_var_=str_var(ndx_sel_var);
for ih=1:length(ndx_sel_hor)
    ih_=ndx_sel_hor(ih);
    testm=squeeze(repfevda(ndx_sel_var,ih_,:));
    column_label=str_sou;
    row_label=str_var_;
    input.data=testm;
    input.tableColLabels =column_label;
    input.tableRowLabels =row_label;
    input.transposeTable = 0;
    % Determine whether input.dataFormat is applied column or row based:
    input.dataFormatMode = 'column'; % use 'column' or 'row'. if not set 'colum' is used
    input.dataFormat = {'%2.1f',size(testm,2)}; % three digits precision for first two columns, one digit for the last
    % Define how NaN values in input.tableData should be printed in the LaTex table:
    input.dataNanString = '-';
    % Column alignment in Latex table ('l'=left-justified,'c'=centered,'r'=right-justified):
    input.tableColumnAlignment = 'r';
    % Switch table borders on/off:
    input.tableBorders = 1;
    % LaTex table caption:
    input.tableCaption =['Variance decomposition ' int2str(ih_) ' steps ahead'];
    % LaTex table label:
    input.tableLabel = ['FEVD_table_' int2str(ih_)];
    % Switch to generate a complete LaTex document or just a table:
    input.makeCompleteLatexDocument = 0;
    % call latexTable:
    latex = latexTable(input);
    latex=strrep(latex,'H!','htbp');
    %latex=strrep(latex,'{table}','{sidewaystable}');
    %latex=strrep(latex,'_','\_');
    if exist([input.tableLabel '.tex'],'file')
        system(['rm ' input.tableLabel '.tex']);
    end
    diary([input.tableLabel '.tex']);
    disp(char(latex));
    diary off;
end
system('mv *.tex ../output');
