% code to make Table_1 in the paper (numbering can change but this is ref 
% Gianni 10.09.2021
% load parameter estimates for vp2, short (6609) and long sample (6619)
% full sample and 1966-2009
clear; clc; close all;
if isunix & ismac==0
    addpath(genpath('/cmc/scratch/m1gga00/matlab_code'));
elseif ismac    
    addpath(genpath('/Users/giovanniamisano/Documents/gianni/ricerca/matlab_code'));
end
%% load all estimates
Datesv=(1966:.25:2009)';
Datesv_new=(1966:.25:2019)';
%%
str_models={'1966-2009','1966-2019'};
str_models_= strrep(str_models,'-','_');
%%
% LOAD strings describing where results sit;
results_location_str = ... 
    { ...
    '../../../RATTS_2020_asy_prior_2021/vp2/estimation_1966_2009_missing_short_rate/RATTS_2020_results.mat', ...
    '../../../RATTS_2020_asy_prior_2021/vp2/estimation_full_sample_missing_short_rate/RATTS_2020_results.mat', ...
    };
nmod=numel(results_location_str); % defines number of models 
%% now define recipients and load results (might be slow) 
M=2e+4;
ngrid=1e+2; % number of points in the grid to compute kernel estimates for prior - posterior
npar=45; % number of elements stored in param
np=40; % number of parameters in the model
nsimpar=34; % number of parameters being actually simulated
priorparama=zeros(M,np,nmod);
parama=zeros(M,npar,nmod); % storing the paramet draws for each model 
% notice that param includes also lnprior lnl accpt prob etc, hence npar =
% 45, not np = 40;
smpllimv=zeros(2,nmod); % to store sample size start and end for each of the models
for imod = 1:nmod
    eval(['load ' results_location_str{imod} ';']);
    smpllimv(:,imod)=datesv([1 end]);
    % store parameters 
    priorparama(:,:,imod)=priorparam;
    parama(:,:,imod)=param;
    clc;
    ['loading model ' num2str(imod) ' of ' num2str(nmod)]
%
end
% now ordering of the models is
% 1 v2p 66-09
% 2 v2p 66-19
% this is to define row labels for latex table
str_par_names_latex= { ...
'$p_{\pi^*_{11}}$','$p_{\pi^*_{00}}$','$p_{\eta,11}$','$p_{\eta,00}$','$p_{z,11}$','$p_{z,00}$' ...
'$\pi^*_1 $','$\pi^*_0 $','$\sigma_{\eta,1} $','$\sigma_{\eta,0} $','$\sigma_{z,1} $','$\sigma_{z,0} $' ...
'$\sigma_{\mu}$','$\sigma_{\xi} $','$\sigma_G $','$\rho_{\mu}$','$\rho_z$','$\rho_G$' ...
'$\rho_{\xi}$','$\psi_{\pi}$','$\psi_y$','$\rho_I$','$\xi  $','$\iota$' ...
'$\phi$','$\alpha$','$\gamma$','$\psi$','$\zeta$','$h$' ...
'$\theta$','$100 \times (1/ \beta-1)$','$g$','$\mu_w$','$\sigma_{me,\pi} $','$\sigma_{me,\Delta c} $' ...
'$\sigma_{me,\Delta y} $','$\sigma_{me,i} $','$\sigma_{me,i12} $','$\sigma_{me,i40} $'           ...
};
str_par_names_latex_=strrep(str_par_names_latex,'$','');
% transform std and growth rates in pa term 
% keep old parama (without transformations) and store it into parama_
%
priorparama_=priorparama;
priorparama(:,9:15,:)=100*priorparama(:,9:15,:);
priorparama(:,32,:)=100*(1./priorparama(:,32,:)-1);
priorparama(:,35:40,:)=1e4*priorparama(:,35:40,:);
priorparama(:,[7 8 23],:)=100*(priorparama(:,[7 8 23],:)-1);
%
parama_=parama;
parama(:,9:15,:)=100*parama(:,9:15,:);
parama(:,32,:)=100*(1./parama(:,32,:)-1);
parama(:,35:40,:)=1e4*parama(:,35:40,:);
parama(:,[7 8 23],:)=100*(parama(:,[7 8 23],:)-1);
%
qv=[(1-clev)/2; 1-((1-clev)/2)];
param_tablea=mean_std_quantiles(permute(parama,[2 3 1]),qv);
priorparam_tablea=mean_std_quantiles(permute(priorparama,[2 3 1]),qv);


% now chart with the short and long sample parameter estimates
nfig=1;
subsetv=[1 2]; %  compare the 66-09 and 66-19 models
labels=str_models(subsetv);
figure(nfig); 
for ip=1:numel(sim1ndx)
    ip1=sim1ndx(ip);
    parm=squeeze(parama(:,ip1,subsetv));
    subplot(7,5,ip);
    boxplot(parm,'Notch','on','Labels',labels,'OutlierSize',0.01)
    %hist(parm,50);
    title(str_par_names_latex_(ip1));
end
sgtitle('Parameter estimates, short (66-09) and long sample (66-19)')
orient(nfig,'portrait')
print(nfig,'-fillpage','../output/par_compare_vp2_6609_6619','-dpdf');



% now charts with posteriors from both models
nfig=nfig+1;
figure(nfig);
% loop over the parameters
for ip=1:numel(sim1ndx)
    ip1=sim1ndx(ip);
    parm=squeeze(parama(:,ip1,:));
    xlimm=sort(parm);
    xlimm=xlimm(round(.01*M):round(.99*M),:);
    pts = linspace(min(xlimm(:)),max(xlimm(:)),100);
    % points to evaluate the estimator
    ksm=zeros(ngrid,2);
    for ic=1:2
        ksm(:,ic) = ksdensity(parm(:,ic),pts);
    end
    subplot(7,5,ip);
    plot1=plot(pts,ksm);
    set(plot1(2),'LineWidth',3);
    set(plot1(2),'LineWidth',2);
    title(str_par_names_latex_(ip1));
    if ip==1
        legend('6609','6619','Location','NorthWest','NumColumns',2);
        legend boxoff;
    end
end
sgtitle('Posterior distributions in the two samples')
orient(nfig,'portrait')
print(nfig,'-fillpage','../output/posteriors_6609_6619','-dpdf');




% now charts with prior and posteriors 
for imod=1:nmod
    nfig=nfig+1;
    figure(nfig);
    % loop over the parameters 
    for ip=1:numel(sim1ndx)
        ip1=sim1ndx(ip);
        parm=[ squeeze(priorparama(:,ip1,imod)) squeeze(parama(:,ip1,imod))];
        xlimm=sort(parm);
        xlimm=xlimm(round(.01*M):round(.99*M),:);
        pts = linspace(min(xlimm(:)),max(xlimm(:)),100);
        % points to evaluate the estimator
        ksm=zeros(ngrid,2);
        for ic=1:2
            ksm(:,ic) = ksdensity(parm(:,ic),pts);
        end
        subplot(7,5,ip);
        plot1=plot(pts,ksm);
        set(plot1(2),'LineWidth',3);
        title(str_par_names_latex_(ip1));
    end
    sgtitle('Priors and posteriors')
    orient(nfig,'portrait')
    print(nfig,'-fillpage',['../output/priors_posteriors_' str_models_{imod}] ,'-dpdf');
end


%% now prepare the tables
% we will have table 1a (structural parameters) and table 1b (measurement
% errors), for both short and long sample

testm_str={...
    'post. mean      ', ...
    'post. std       ', ...
    'post. 2.5       ', ...
    'post. 97.5      ', ...
    'prior mean      ', ...
    'prior std       ', ...
    'prior 2.5 \%    ', ...
    'prior 97.5 \%   ', ...
    };

for imod=1:nmod
    testm=[squeeze(priorparam_tablea(sim1ndx,imod,:))  squeeze(param_tablea(sim1ndx,imod,:))];
    column_label=testm_str(1:8);
    row_label=str_par_names_latex(sim1ndx);
    input.data=testm;
    input.tableColLabels =column_label;
    input.tableRowLabels =row_label;
    input.transposeTable = 0;
    % Determine whether input.dataFormat is applied column or row based:
    input.dataFormatMode = 'column'; % use 'column' or 'row'. if not set 'colum' is used
    input.dataFormat = {'%2.2f',8}; % three digits precision for first two columns, one digit for the last
    % Define how NaN values in input.tableData should be printed in the LaTex table:
    input.dataNanString = '-';
    % Column alignment in Latex table ('l'=left-justified,'c'=centered,'r'=right-justified):
    input.tableColumnAlignment = 'r';
    % Switch table borders on/off:
    input.tableBorders = 1;
    % LaTex table caption:
    input.tableCaption = ['Estimation output ' str_models{imod}];
    % LaTex table label:
    input.tableLabel = ['EstimationOutput_' str_models_{imod}];
    % Switch to generate a complete LaTex document or just a table:
    input.makeCompleteLatexDocument = 0;
    % call latexTable:
    latex = latexTable(input);
    latex=strrep(latex,'H!','htbp');
    latex=strrep(latex,'{table}','{sidewaystable}');
    %latex=strrep(latex,'_','\_');
    if exist([input.tableLabel '.tex'],'file')
        system(['rm ' input.tableLabel '.tex']);
    end
    diary([input.tableLabel '.tex']);
    disp(char(latex));
    diary off;
    system('mv *.tex ../output');
end

% now pre tables for prior and posterior separately 
for imod=1:nmod
    % prior 
    testm=squeeze(priorparam_tablea(sim1ndx,imod,:));
    column_label=testm_str(5:8);
    row_label=str_par_names_latex(sim1ndx);
    input.data=testm;
    input.tableColLabels =column_label;
    input.tableRowLabels =row_label;
    input.transposeTable = 0;
    % Determine whether input.dataFormat is applied column or row based:
    input.dataFormatMode = 'column'; % use 'column' or 'row'. if not set 'colum' is used
    input.dataFormat = {'%2.2f',size(testm,2)}; 
    % Define how NaN values in input.tableData should be printed in the LaTex table:
    input.dataNanString = '-';
    % Column alignment in Latex table ('l'=left-justified,'c'=centered,'r'=right-justified):
    input.tableColumnAlignment = 'r';
    % Switch table borders on/off:
    input.tableBorders = 1;
    % LaTex table caption:
    input.tableCaption = ['Prior specification ' str_models{imod}];
    % LaTex table label:
    input.tableLabel = ['Prior_' str_models_{imod}];
    % Switch to generate a complete LaTex document or just a table:
    input.makeCompleteLatexDocument = 0;
    % call latexTable:
    latex = latexTable(input);
    latex=strrep(latex,'H!','htbp');
    %latex=strrep(latex,'{table}','{sidewaystable}');
    %latex=strrep(latex,'_','\_');
    if exist([input.tableLabel '.tex'],'file')
        system(['rm ' input.tableLabel '.tex']);
    end
    diary([input.tableLabel '.tex']);
    disp(char(latex));
    diary off;
    system('mv *.tex ../output');
    %
    % Posterior
    %
    testm=squeeze(param_tablea(sim1ndx,imod,:));
    column_label=testm_str(1:4);
    row_label=str_par_names_latex(sim1ndx);
    input.data=testm;
    input.tableColLabels =column_label;
    input.tableRowLabels =row_label;
    input.transposeTable = 0;
    % Determine whether input.dataFormat is applied column or row based:
    input.dataFormatMode = 'column'; % use 'column' or 'row'. if not set 'colum' is used
    input.dataFormat = {'%2.2f',size(testm,2)}; 
    % Define how NaN values in input.tableData should be printed in the LaTex table:
    input.dataNanString = '-';
    % Column alignment in Latex table ('l'=left-justified,'c'=centered,'r'=right-justified):
    input.tableColumnAlignment = 'r';
    % Switch table borders on/off:
    input.tableBorders = 1;
    % LaTex table caption:
    input.tableCaption = ['Posterior estimates ' str_models{imod}];
    % LaTex table label:
    input.tableLabel = ['Posterior_' str_models_{imod}];
    % Switch to generate a complete LaTex document or just a table:
    input.makeCompleteLatexDocument = 0;
    % call latexTable:
    latex = latexTable(input);
    latex=strrep(latex,'H!','htbp');
    %latex=strrep(latex,'{table}','{sidewaystable}');
    %latex=strrep(latex,'_','\_');
    if exist([input.tableLabel '.tex'],'file')
        system(['rm ' input.tableLabel '.tex']);
    end
    diary([input.tableLabel '.tex']);
    disp(char(latex));
    diary off;
    system('mv *.tex ../output');
    
end







