function [retcode,y_,gx,hx,gxx,hxx,gss,hss,eta,etastd,nfgrad,nfhess] = RATTS_2020_modsol_Klein_check(order,deltav,n1,n2,n3,n4)
% Solves the RATTS model with regime switches
%
% The variables are ordered as follows
% x = [lag_I lag_Pi lag_C lag_Y xi_t z_t pol_t mu_t G_t st_pist_t];
% y = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY]; 
n1_=n1+1;
%====================================================
% Input parameters
%====================================================
p_pist_1=	deltav(	1	,1	)	;
p_pist_0=	deltav(	2	,1	)	;
p_pol_1	=	deltav(	3	,1	)	;
p_pol_0	=	deltav(	4	,1	)	;
p_z_1	=	deltav(	5	,1	)	;
p_z_0	=	deltav(	6	,1	)	;
lngtrg_1=	deltav(	7	,1	)	;
lngtrg_0=	deltav(	8	,1	)	;
stdpol_1	=	deltav(	9	,1	)	;
stdpol_0	=	deltav(	10	,1	)	;
stdz_1	=	deltav(	11	,1	)	;
stdz_0	=	deltav(	12	,1	)	;
stdmu	=	deltav(	13	,1	)	;
stdxi	=	deltav(	14	,1	)	;
stdG    =   deltav(	15	,1	)	;    
rhomu	=	deltav(	16	,1	)	;
rhoz	=	deltav(	17	,1	)	;
rhoG	=	deltav(	18	,1	)	;
rhoxi	=	deltav(	19	,1	)	;
deltPi	=	deltav(	20	,1	)	;
deltY	=	deltav(	21	,1	)	;
rhoint	=	deltav(	22	,1	)	;
xi	=	deltav(	23	,1	)	;
iot	=	deltav(	24	,1	)	;
phi 	=	deltav(	25	,1	)	;
alf	=	deltav(	26	,1	)	;
gam	=	deltav(	27	,1	)	;
psai	=	deltav(	28	,1	)	;
zet	=	deltav(	29	,1	)	;
h 	=	deltav(	30	,1	)	;
thet	=	deltav(	31	,1	)	;
bet	=	deltav(	32	,1	)	;
g	=	deltav(	33	,1	)	;
muw	=	deltav(	34	,1	)	;
%====================================================
%Calculate the nonstochastic steady state of all variables in levels
%====================================================

etaL = 1/(1/3)^(1+1/phi)/(1 - psai + thet/(thet-1)/alf*psai*(1+muw)*(1+1/phi)*(1-h)*(1-g)) ; % scaling disutility of labour

% ================================================
st_pist_t = (1 - p_pist_0)/(2 - p_pist_1 - p_pist_0) ;
% ================================================

% Start from the levels
xi_t = log(xi) ;
z_t = 0 ;
mu_t = muw ;
pol_t = 0 ;
Pi_t = lngtrg_1*st_pist_t + lngtrg_0*(1 - st_pist_t) ;
I_t = Pi_t/bet*xi^psai ; 
lag_I = I_t ; 
lag_Pi = Pi_t ;
Y_t = etaL^(-alf*phi/(phi+1))/(1 - psai + thet/(thet-1)/alf*psai*(1+muw)*(1+1/phi)*(1-h)*(1-g))^(alf*phi/(phi+1)) ;
Y_SS = Y_t ;
G_t = g*Y_SS;
C_t = (1-g)*Y_SS;
lag_C = C_t ;
J_t = ((1-bet)/(1-bet*xi^(1-psai)))^(1/(1-psai))*(1-h)*(1-g)*Y_t*(1 - etaL*(1-psai)*Y_t^(1/alf*(1+1/phi)))^(psai/(1-psai)) ;

scaleJ = J_t*.9; 

Lam_t = C_t^(-psai)*(1-h)^(-psai)*(1 - etaL*(1-psai)*Y_t^(1/alf*(1+1/phi)))^psai ; 
dum_t = (xi*J_t/scaleJ)^(1-gam) ;
deltaC = xi ;
lag_Y = Y_t;
deltaY = xi ;

%Take logs (NOT FOR THE SWITCHING STATES)
% J_t = log(1-h)+log(Y_t)+log(Lbar - Y_t^(phi/alf)) ;
% dum_t = (1-gam)*J_t ;
mu_t = log(mu_t) ;
G_t = log(G_t); 
Pi_t = log(Pi_t);
I_t = log(I_t); 
lag_I = log(lag_I) ; 
lag_Pi = log(lag_Pi) ;
Y_t = log(Y_t) ;
C_t = log(C_t);
lag_C = log(lag_C);
Lam_t = log(Lam_t); 
J_t = log(J_t) ;
dum_t = log(dum_t) ;
deltaC = log(deltaC) ;
lag_Y = log(lag_Y) ;
deltaY = log(deltaY) ;
%
% NSSS values
y_ = [Pi_t C_t J_t Y_t I_t Lam_t dum_t deltaC deltaY]';
%
%Next-period variables
xi_tp = xi_t ;
%z_tp = z_t ;
%mu_tp = mu_t ;
%G_tp = G_t; 
%pol_tp = pol_t ;
Pi_tp = Pi_t;
%I_tp = I_t; 
lag_Ip = lag_I ; 
lag_Pip = lag_Pi ;
%C_tp = C_t;
Y_tp = Y_t ;
lag_Cp = lag_C;
J_tp = J_t ;
Lam_tp = Lam_t; 
%dum_tp = dum_t ;
%deltaCp = deltaC ;
lag_Yp = lag_Y ;
%deltaYp = deltaY ;
%st_pist_tp = st_pist_t;
%====================================================
% Define matrices of dynamic equations
%====================================================
% Note: Klein's order is: xp,yp,x,y
% 
% Note: Klein's order is: xp,yp,x,y

nfgrad=[0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhoz,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhomu,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhoG,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-lag_C)*exp(C_t)*exp(xi_t),0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,-exp(deltaC),0;
0,0,0,exp(lag_Yp),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(Y_t),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-lag_Y)*exp(Y_t)*exp(xi_t),exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0,0,0,0,exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,-exp(deltaY);
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhoint*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,0,deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(psai*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,-exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,(exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,-(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)),0,0,0,exp(I_t),0,0,0,-deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint);
0,0,0,0,(exp(J_tp)*exp(xi_tp)*(gam-1))/(scaleJ*((exp(J_tp)*exp(xi_tp))/scaleJ)^gam),0,0,0,0,0,0,0,(exp(J_tp)*exp(xi_tp)*(gam-1))/(scaleJ*((exp(J_tp)*exp(xi_tp))/scaleJ)^gam),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(dum_t),0,0;
0,0,0,0,(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,0,0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-I_t),(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(h*psai*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai)/(exp(C_t)-h*exp(lag_C))^(psai+1),0,0,-(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,0,0,-(psai*exp(C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai)/(exp(C_t)-h*exp(lag_C))^(psai+1),0,(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,-exp(Lam_t),0,0,0;
0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),(etaL*h*psai*thet*exp(-Lam_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi),0,(etaL*psai*thet*exp(-Lam_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot)-(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,-zet*exp(2*Pi_t)*exp(Y_t)-zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),-(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2-zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))-exp(Y_t)*(thet-1)+(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1))-(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
exp(lag_Ip),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(I_t),0,0,0,0;
0,exp(lag_Pip),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(Pi_t),0,0,0,0,0,0,0,0;
0,0,exp(lag_Cp),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(C_t),0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(h*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)),0,0,-(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)),0,0,0,0,0,-(exp(C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)),-exp(J_t),(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)),0,0,-(bet*scaleJ^(1-psai)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1))/((bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,0,0,0,0,exp(G_t),(zet*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),exp(C_t),0,(zet*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))^2)/2-exp(Y_t),0,0,0,0,0;
0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhoxi,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1-p_pist_1-p_pist_0,0,0,0,0,0,0,0,0,0];

nfhess=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,-exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,-exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-lag_C)*exp(C_t)*exp(xi_t),0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-lag_C)*exp(C_t)*exp(xi_t),0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,exp(-lag_C)*exp(C_t)*exp(xi_t),0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(deltaC),0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,exp(lag_Yp),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(Y_t),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(-lag_Y)*exp(Y_t)*exp(xi_t),-exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0,0,0,0,-exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-lag_Y)*exp(Y_t)*exp(xi_t),exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0,0,0,0,exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(-lag_Y)*exp(Y_t)*exp(xi_t),exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0,0,0,0,exp(-lag_Y)*exp(Y_t)*exp(xi_t),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(deltaY);
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhoint*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)-rhoint*exp(2*lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-2)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,0,deltY*rhoint*exp(-xi_t)*exp(deltaY)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(psai*rhoint*exp(lag_I)*exp(pol_t)*exp(xi_t)*exp(lag_I)^(rhoint-1)*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,-rhoint*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,(rhoint*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltPi*rhoint*exp(Pi_t)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,-(deltPi*rhoint*exp(Pi_t)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)),0,0,0,0,0,0,0,-deltY*rhoint*exp(-xi_t)*exp(deltaY)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint);
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,deltY*rhoint*exp(-xi_t)*exp(deltaY)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(psai*rhoint*exp(lag_I)*exp(pol_t)*exp(xi_t)*exp(lag_I)^(rhoint-1)*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,0,0,(psai*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-deltY*exp(2*deltaY)*exp(-2*xi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-2)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(deltY-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)-deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(psai*exp(2*xi_t)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-2)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(psai-1)*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(psai^2*rhoint*exp(2*xi_t)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^(2*psai-2)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet^2*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(rhoint+1))-(2*deltY*psai*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(psai*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,0,(psai*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2-(deltPi*psai*exp(Pi_t)*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(psai*rhoint*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet^2*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(rhoint+1)),(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))+(deltPi*psai*exp(Pi_t)*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,0,0,0,0,0,0,deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+deltY*exp(2*deltaY)*exp(-2*xi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-2)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(deltY-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(deltY*psai*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint);
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-rhoint*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,0,deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(psai*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,-exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,(exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,-(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)),0,0,0,0,0,0,0,-deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint);
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(rhoint*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltPi*rhoint*exp(Pi_t)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,0,0,0,(psai*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2-(deltPi*psai*exp(Pi_t)*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)-(psai*rhoint*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet^2*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(rhoint+1)),0,(exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,0,0,-(2*deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)^2*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^3-(rhoint*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^(2*psai)*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)^2*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet^2*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(rhoint+1))-(deltPi*exp(2*Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)^2*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-2)*(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^4-(2*deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)^2*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2+(deltPi*exp(2*Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-2)*(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^3+(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,0,0,0,0,0,0,(deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(deltPi*rhoint*exp(Pi_t)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)),0,0,0,(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))+(deltPi*psai*exp(Pi_t)*exp(pol_t)*exp(xi_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,-(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)),0,0,(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2+(deltPi*exp(2*Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-2)*(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^3+(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^deltY*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*(rhoint-1))/(bet*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),-(deltPi*exp(Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))-(deltPi*exp(2*Pi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^deltY*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-2)*(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,0,0,0,0,0,0,0,-(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1));
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(I_t),0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-deltY*rhoint*exp(-xi_t)*exp(deltaY)*exp(lag_I)*exp(pol_t)*exp(lag_I)^(rhoint-1)*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,0,deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+deltY*exp(2*deltaY)*exp(-2*xi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-2)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(deltY-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)+(deltY*psai*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^(psai-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint),0,-deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint),0,0,(deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*exp(xi_t)^psai*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(rhoint-1))/(bet*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^rhoint)+(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(lngtrg_1-lngtrg_0)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^2,-(deltY*deltPi*exp(-xi_t)*exp(Pi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^(deltPi-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)),0,0,0,0,0,0,0,-deltY*exp(-xi_t)*exp(deltaY)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-1)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint)-deltY*exp(2*deltaY)*exp(-2*xi_t)*exp(pol_t)*exp(lag_I)^rhoint*(exp(-xi_t)*exp(deltaY))^(deltY-2)*(exp(Pi_t)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))^deltPi*(deltY-1)*((exp(xi_t)^psai*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1)))/bet)^(1-rhoint);
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(exp(J_tp)*exp(xi_tp)*(gam-1))/(scaleJ*((exp(J_tp)*exp(xi_tp))/scaleJ)^gam)-(gam*exp(2*J_tp)*exp(2*xi_tp)*(gam-1))/(scaleJ^2*((exp(J_tp)*exp(xi_tp))/scaleJ)^(gam+1)),0,0,0,0,0,0,0,(exp(J_tp)*exp(xi_tp)*(gam-1))/(scaleJ*((exp(J_tp)*exp(xi_tp))/scaleJ)^gam)-(gam*exp(2*J_tp)*exp(2*xi_tp)*(gam-1))/(scaleJ^2*((exp(J_tp)*exp(xi_tp))/scaleJ)^(gam+1)),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(exp(J_tp)*exp(xi_tp)*(gam-1))/(scaleJ*((exp(J_tp)*exp(xi_tp))/scaleJ)^gam)-(gam*exp(2*J_tp)*exp(2*xi_tp)*(gam-1))/(scaleJ^2*((exp(J_tp)*exp(xi_tp))/scaleJ)^(gam+1)),0,0,0,0,0,0,0,(exp(J_tp)*exp(xi_tp)*(gam-1))/(scaleJ*((exp(J_tp)*exp(xi_tp))/scaleJ)^gam)-(gam*exp(2*J_tp)*exp(2*xi_tp)*(gam-1))/(scaleJ^2*((exp(J_tp)*exp(xi_tp))/scaleJ)^(gam+1)),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(dum_t),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1))-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(2*xi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam)*(gam+1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+2)),0,0,0,0,0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^(gam+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam)-(bet*scaleJ^(gam-psai)*exp(2*J_tp)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-2)*(gam-psai)*(gam-psai+1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam-1)*(gam-psai)^2)/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,0,0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(-I_t),0,0,0,0;
0,0,0,0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1)),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(J_tp)^(psai-gam))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0;
0,0,0,0,-(bet*gam*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^(gam+1)*(gam-1)),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(J_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam-1)*(gam-psai)^2)/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0,(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1)),(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(Lam_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam*(gam-1))-(bet*scaleJ^(gam-psai)*exp(-Lam_t)*exp(-Pi_tp)*exp(2*dum_t)*exp(Lam_tp)*exp(J_tp)^(psai-gam)*((gam-psai)/(gam-1)+1)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+2)*exp(xi_tp)^gam*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(h*psai*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai)/(exp(C_t)-h*exp(lag_C))^(psai+1)+(h^2*psai*exp(2*lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(psai+1))/(exp(C_t)-h*exp(lag_C))^(psai+2),0,0,-(etaL*h*psai^2*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,0,0,0,-(h*psai*exp(C_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(psai+1))/(exp(C_t)-h*exp(lag_C))^(psai+2),0,(etaL*h*psai^2*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(etaL*h*psai^2*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai)+(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,0,0,(etaL*psai^2*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,-(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai)-(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(h*psai*exp(C_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(psai+1))/(exp(C_t)-h*exp(lag_C))^(psai+2),0,0,(etaL*psai^2*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,0,0,0,(psai*exp(2*C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(psai+1))/(exp(C_t)-h*exp(lag_C))^(psai+2)-(psai*exp(C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai)/(exp(C_t)-h*exp(lag_C))^(psai+1),0,-(etaL*psai^2*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(etaL*h*psai^2*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,-(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai)-(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,0,0,-(etaL*psai^2*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai)+(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((1/phi+1)/alf-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(Lam_t),0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(bet*gam*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(2*xi_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^(gam+1))-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*(lngtrg_1-lngtrg_0)*(gam-1)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai))/exp(dum_t)^((gam-psai)/(gam-1)),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam-1)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai))/exp(dum_t)^((gam-psai)/(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(2*J_tp)*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-2)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-psai+1))/exp(dum_t)^((gam-psai)/(gam-1))-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(gam-psai)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(gam-psai))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)^2*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,0,0,0,0,0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,0,0,0,0,0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)+iot*zet*exp(2*lag_Pi)*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^(iot-2)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(iot-1),0,0,0,0,0,0,0,-(iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),0,0,iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(etaL*h*psai*thet*exp(-Lam_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*h^2*psai^2*thet*exp(-Lam_t)*exp(2*lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,-(etaL^2*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,(etaL*h*psai*thet*exp(-Lam_t)*exp(lag_C)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,-(etaL*h*psai^2*thet*exp(-Lam_t)*exp(C_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,(etaL*h*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL^2*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,-(etaL*h*psai*thet*exp(-Lam_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(etaL^2*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,0,(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)+(etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(psai-1)^2)/alf^2+(etaL^3*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-3)*(1/phi+1)^3*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^3*(psai-2))/alf^3+(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/alf-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)+(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(2/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-2)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)+(2*etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/(alf^3*phi),0,-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2,0,0,0,(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2-(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-z_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)-(etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-z_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^3-(etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(psai-1)^2)/alf^2-(etaL^3*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-3)*(1/phi+1)^3*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^3*(psai-2))/alf^3-(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/alf-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)-(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(2/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-2)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)-(2*etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/(alf^3*phi),0,(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi),0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(etaL*h*psai*thet*exp(-Lam_t)*exp(lag_C)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2,0,(etaL*psai*thet*exp(-Lam_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf,0,0,0,-(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)+(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2,0,-(etaL*psai*thet*exp(-Lam_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*(lngtrg_1-lngtrg_0)*(gam-1)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(gam-psai)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),0,0,0,0,-(iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,0,0,0,0,0,0,0,(iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)^2*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(iot+1)-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)^2*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(iot+1)),(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot)-(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,0,0,-(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(gam-psai)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot*(gam-1)),0,0;
0,0,0,0,(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,0,0,(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(gam-psai))/exp(dum_t)^((gam-psai)/(gam-1)),-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),0,0,0,0,0,0,0,(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot)-(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,-3*zet*exp(2*Pi_t)*exp(Y_t)-zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1))-(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(2*Pi_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-2)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(iot-1))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,-zet*exp(2*Pi_t)*exp(Y_t)-zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(etaL*h*psai^2*thet*exp(-Lam_t)*exp(C_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1)),0,0,(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,-(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0,(etaL*psai^2*thet*exp(2*C_t)*exp(-Lam_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^(psai+1))-(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,-(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),(etaL*h*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL^2*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)+(etaL*h*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,0,-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2-(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-z_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)-(etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-z_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^3-(etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(psai-1)^2)/alf^2-(etaL^3*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-3)*(1/phi+1)^3*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^3*(psai-2))/alf^3-(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/alf-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)-(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(2/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-2)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)-(2*etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/(alf^3*phi),0,(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)+(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2,0,-(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,-zet*exp(2*Pi_t)*exp(Y_t)-zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),-(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1)^3)/(alf^2*(exp(C_t)-h*exp(lag_C))^psai)-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(C_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(psai-1))/(alf^2*phi*(exp(C_t)-h*exp(lag_C))^psai),0,(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2-zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))-exp(Y_t)*(thet-1)+(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(Y_t)^(1/alf-2)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/alf-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2+(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2+(2*etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-z_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)+(2*etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-z_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^3+(etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(psai-1)^2)/alf^2+(etaL^3*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-3)*(1/phi+1)^3*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^3*(psai-2))/alf^3+(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/alf-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi)+(etaL*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(2/alf-2)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-2)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^3*phi)+(2*etaL^2*psai*thet*exp(-Lam_t)*exp(2*Y_t)*exp(-2*z_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/(alf^3*phi),0,-(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi),0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))*(gam-1))/(exp(dum_t)^((gam-psai)/(gam-1))*exp(xi_tp)^gam),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam))/exp(dum_t)^((gam-psai)/(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1)),0,0,0,0,0,-(etaL*h*psai*thet*exp(-Lam_t)*exp(lag_C)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,0,(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2+(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi),0,-(etaL*psai*thet*exp(-Lam_t)*exp(mu_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1))*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot),(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))/exp(dum_t)^((gam-psai)/(gam-1)),(etaL*psai*thet*exp(-Lam_t)*exp(C_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai),0,-(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)*exp(Y_t)^(1/alf-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf^2-(etaL^2*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(psai-1)^2)/alf^2-(etaL*psai*thet*exp(-Lam_t)*exp(-z_t)*exp(Y_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*(exp(-z_t)*exp(Y_t))^(1/alf-1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi-1)*(exp(C_t)-h*exp(lag_C))^(1-psai))/(alf^2*phi),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/exp(dum_t)^((gam-psai)/(gam-1))+(etaL*psai*thet*exp(-Lam_t)*exp(Y_t)^(1/alf)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(mu_t)+1)*((exp(-z_t)*exp(Y_t))^(1/alf))^(1/phi)*(exp(C_t)-h*exp(lag_C))^(1-psai))/alf,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(xi_tp)*exp(J_tp)^(psai-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*exp(xi_tp)^gam),0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(J_tp)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam-1)*exp(xi_tp)^(1-gam)*(gam-psai)^2*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0,0,0,0,0,0,0,0,0,0,0,-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(Pi_t)^iot*exp(xi_tp)^(1-gam)*(gam-psai)*(lngtrg_1-lngtrg_0)*(iot-1))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot*(gam-1)),(bet*iot*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Pi_t)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(Pi_t)^(iot-1)*exp(xi_tp)^(1-gam)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(gam-psai))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0,0,0,(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(2*dum_t)*exp(Lam_tp)*exp(Y_tp)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*((gam-psai)/(gam-1)+1)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+2)*(gam-1))-(bet*scaleJ^(gam-psai)*zet*exp(-Lam_t)*exp(Lam_tp)*exp(Y_tp)*exp(dum_t)*exp(J_tp)^(psai-gam)*exp(xi_tp)^(1-gam)*(gam-psai)*(exp(Pi_tp)-exp(Pi_t)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(exp(dum_t)^((gam-psai)/(gam-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
exp(lag_Ip),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(I_t),0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,exp(lag_Pip),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(Pi_t),0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,exp(lag_Cp),0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(C_t),0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(h*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(h^2*psai*exp(2*lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^(psai+1)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(h^2*exp(2*lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai)*(1/(psai-1)+1)*(bet-1)^2*(psai-1))/((exp(C_t)-h*exp(lag_C))^(2*psai)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,-(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,0,0,0,-(h*psai*exp(C_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^(psai+1)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(h*exp(C_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai)*(1/(psai-1)+1)*(bet-1)^2*(psai-1))/((exp(C_t)-h*exp(lag_C))^(2*psai)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,-(bet*h*scaleJ^(1-psai)*exp(dum_t)*exp(lag_C)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/(psai-1)+1)*(bet-1)*(psai-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1)^2)/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL^2*psai^2*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(2-2*psai)*(bet-1)^2*(psai-1))/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,0,0,0,(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,-(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1)^2)/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL^2*psai^2*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(2-2*psai)*(bet-1)^2*(psai-1))/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,(bet*etaL*psai*scaleJ^(1-psai)*exp(-z_t)*exp(Y_t)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(h*psai*exp(C_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^(psai+1)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(h*exp(C_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai)*(1/(psai-1)+1)*(bet-1)^2*(psai-1))/((exp(C_t)-h*exp(lag_C))^(2*psai)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,0,0,0,(psai*exp(2*C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^(psai+1)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(exp(C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(bet-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(exp(2*C_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai)*(1/(psai-1)+1)*(bet-1)^2*(psai-1))/((exp(C_t)-h*exp(lag_C))^(2*psai)*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,-(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,(bet*scaleJ^(1-psai)*exp(C_t)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/(psai-1)+1)*(bet-1)*(psai-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-exp(J_t),0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL*h*psai*exp(-z_t)*exp(Y_t)*exp(lag_C)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,-(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1)^2)/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL^2*psai^2*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(2-2*psai)*(bet-1)^2*(psai-1))/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,0,0,0,-(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(bet-1)*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))-(etaL*psai*exp(-z_t)*exp(C_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)^2*(psai-1))/(alf*(exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,(etaL*psai*exp(-z_t)*exp(Y_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL^2*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1)^2)/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL*psai*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-2)*(exp(C_t)-h*exp(lag_C))^(1-psai)*((1/phi+1)/alf-1)*(bet-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1))+(etaL^2*psai^2*exp(2*Y_t)*exp(-2*z_t)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(2*psai-2)*(1/phi+1)^2*(exp(-z_t)*exp(Y_t))^((2*(1/phi+1))/alf-2)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(2-2*psai)*(bet-1)^2*(psai-1))/(alf^2*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)),0,0,-(bet*etaL*psai*scaleJ^(1-psai)*exp(-z_t)*exp(Y_t)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-(bet*h*scaleJ^(1-psai)*exp(dum_t)*exp(lag_C)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/(psai-1)+1)*(bet-1)*(psai-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0,(bet*etaL*psai*scaleJ^(1-psai)*exp(-z_t)*exp(Y_t)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0,0,0,0,(bet*scaleJ^(1-psai)*exp(C_t)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(1/(psai-1)+1)*(bet-1)*(psai-1))/((exp(C_t)-h*exp(lag_C))^psai*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,-(bet*etaL*psai*scaleJ^(1-psai)*exp(-z_t)*exp(Y_t)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1)*(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^(psai-1)*(1/phi+1)*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf-1)*(1/(psai-1)+1)*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1)*(psai-1))/(alf*(bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)),0,0,(bet^2*scaleJ^(2-2*psai)*exp(2*dum_t)*exp(dum_t)^((2*(psai-1))/(gam-1)-2)*(1/(psai-1)+1)*(psai-1))/((bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+2)*(gam-1)^2)-(bet*scaleJ^(1-psai)*exp(2*dum_t)*exp(dum_t)^((psai-1)/(gam-1)-2)*((psai-1)/(gam-1)-1))/((bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)*(gam-1))-(bet*scaleJ^(1-psai)*exp(dum_t)*exp(dum_t)^((psai-1)/(gam-1)-1))/((bet*scaleJ^(1-psai)*exp(dum_t)^((psai-1)/(gam-1))-(etaL*(exp(-z_t)*exp(Y_t))^((1/phi+1)/alf)*(psai-1)+1)^psai*(exp(C_t)-h*exp(lag_C))^(1-psai)*(bet-1))^(1/(psai-1)+1)*(gam-1)),0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,iot^2*zet*exp(2*lag_Pi)*exp(Y_t)*exp(lag_Pi)^(2*iot-2)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(2-2*iot)-iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))-iot*zet*exp(2*lag_Pi)*exp(Y_t)*exp(lag_Pi)^(iot-2)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,0,0,0,0,0,(iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1-lngtrg_0)*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot-(iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^iot*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,-iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),0,0,-iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(G_t),0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,(iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1-lngtrg_0)*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot-(iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^iot*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,0,0,0,0,0,0,0,(zet*exp(Y_t)*exp(lag_Pi)^(2*iot)*(lngtrg_1-lngtrg_0)^2*(iot-1)^2)/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(2*iot)-(iot*zet*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)^2*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(iot+1),(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,0,0,(zet*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-iot*zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot),0,0,0,0,0,0,0,(zet*exp(Pi_t)*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,zet*exp(2*Pi_t)*exp(Y_t)+zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,exp(C_t),0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-iot*zet*exp(Y_t)*exp(lag_Pi)*exp(lag_Pi)^(iot-1)*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,0,0,0,0,0,(zet*exp(Y_t)*exp(lag_Pi)^iot*(lngtrg_1-lngtrg_0)*(iot-1)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)))/(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^iot,zet*exp(Pi_t)*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot)),0,0,(zet*exp(Y_t)*(exp(Pi_t)-exp(lag_Pi)^iot*(lngtrg_1*st_pist_t-lngtrg_0*(st_pist_t-1))^(1-iot))^2)/2-exp(Y_t),0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
%%
%====================================================
% now covariance matrix
%====================================================
% these are discrete shocks: st_pist_t st_pol_t st_z_t (and we follow that
% order)
% ordering of the pages is
% 1) (pist1 pol1 z1)
% 2) (pist1 pol1 z0) 
% 3) (pist1 pol0 z1) 
% 4) (pist1 pol0 z0) 
% 5) (pist0 pol1 z1) 
% 6) (pist0 pol0 z0) 
% 7) (pist0 pol1 z1) 
% 8) (pist0 pol0 z0) 

% Variance-covariance matrix
eta = zeros(10,10,8); % 10 pages of eta: 10 states x 6 shocks x 8 pages
eta(5:10,5:10,1) = diag([stdxi; stdz_1; stdpol_1;stdmu;stdG; sqrt(p_pist_1*(1-p_pist_1))].^2);
eta(5:10,5:10,2) = diag([stdxi; stdz_0; stdpol_1;stdmu;stdG; sqrt(p_pist_1*(1-p_pist_1))].^2);
eta(5:10,5:10,3) = diag([stdxi; stdz_1; stdpol_0;stdmu;stdG; sqrt(p_pist_1*(1-p_pist_1))].^2);
eta(5:10,5:10,4) = diag([stdxi; stdz_0; stdpol_0;stdmu;stdG; sqrt(p_pist_1*(1-p_pist_1))].^2);
eta(5:10,5:10,5) = diag([stdxi; stdz_1; stdpol_1;stdmu;stdG; sqrt(p_pist_0*(1-p_pist_0))].^2);
eta(5:10,5:10,6) = diag([stdxi; stdz_0; stdpol_1;stdmu;stdG; sqrt(p_pist_0*(1-p_pist_0))].^2);
eta(5:10,5:10,7) = diag([stdxi; stdz_1; stdpol_0;stdmu;stdG; sqrt(p_pist_0*(1-p_pist_0))].^2);
eta(5:10,5:10,8) = diag([stdxi; stdz_0; stdpol_0;stdmu;stdG; sqrt(p_pist_0*(1-p_pist_0))].^2);
%
etastd=sqrt(eta(:,5:9,:));
% construct the transition prob matrix of the composite discrete state
tpa=zeros(2,2,3);
tpa(:,:,3)= ... 
    [  p_z_1       1-p_z_1;...
     1-p_z_0         p_z_0];
tpa(:,:,2)= ...
    [  p_pol_1     1-p_pol_1;...
     1-p_pol_0       p_pol_0];
tpa(:,:,1)= ...
    [  p_pist_1    1-p_pist_1;...
     1-p_pist_0      p_pist_0];
PIMAT=kron(tpa(:,:,1),kron(tpa(:,:,2),tpa(:,:,3)))';
%% ====================================================
% 06/07/2019 make sure that the inputs are real
allreal=isreal(nfgrad) & isreal(nfhess) & isreal(PIMAT) & isreal(eta);
%%
if allreal==true
    if order==1
        nfy=nfgrad(:,2*n1_+n2+1:end);
        nfx=nfgrad(:,n1_+n2+1:2*n1_+n2);
        nfyp=nfgrad(:,n1_+1:n1_+n2);
        nfxp=nfgrad(:,1:n1_);
        %retcode
        [retcode,gx,hx] = gx_hx1(nfy,nfx,nfyp,nfxp);
        %retcode
        gxx=zeros(n1_*n2,n1_);
        hxx=zeros(n1_*n1_,n1_);
        gss=zeros(n2,size(etastd,3));
        hss=zeros(n1_,size(etastd,3));
    else
        [retcode,gx,hx,gxx,hxx,hss,gss]=solab2_new_ms_retcode(PIMAT,nfgrad,nfhess,eta,n1_);
       
        
        gx
        hx
        hss
        gss
        
        %{
eta
        PIMAT
        %}
        %    [gx1,hx1,gxx1,hxx1,hss1,gss1]=solab2_ms(nfgrad,nfhess,eta,n1_);
    end
else
    retcode=0;
end
if retcode==0
    hx=zeros(n1_,n1_);
    gx=zeros(n2,n1_);
    gxx=zeros(n1_*n2,n1_);
    hxx=zeros(n1_*n1_,n1_);
    gss=zeros(n2,size(etastd,3));
    hss=zeros(n1_,size(etastd,3));
end
% =========================================================================
% Analytical bond prices and yields
% =========================================================================

% definitions (for comparability with file Euler.tex)
PP = hx ; GG = hxx ; kxs = hss ;

Flam = gx(6,:)' + (psai-gam)/(1-gam)*gx(7,:)';  % lamtil_t
Fpi = gx(1,:)' + (gam-psai)*gx(3,:)' + (psai-gam)/(1-gam)*gx(7,:)' + gam*[0 0 0 0 1 0 0 0 0 0]'; % pi_t

% max maturity used in estimation
max_enne = n4(size(n4,2)) ;
% matrices for the canonical representation
F_bn = zeros(size(hx,1),max_enne);
E_bn = zeros(size(hx,1),size(hx,1),max_enne);
kbns = zeros(max_enne,size(hss,2));
% Matrices for 1-period bonds
F_bn(:,1) = -gx(5,:)' ;
E_bn(:,:,1) = -gxx(4*n1_+1:5*n1_,:) ;
kbns(1,:) = -gss(5,:) ;

for enne = 2:max_enne
    F_bn(:,enne) = (F_bn(:,1)' + F_bn(:,enne-1)'*PP)' ;     % First order
    
    for i = 1:n1_
        E_bn(:,:,enne) = squeeze(E_bn(:,:,enne)) + F_bn(i,enne-1)'*GG(1+n1_*(i-1):n1_+n1_*(i-1),:) ;
    end
    E_bn(:,:,enne) = E_bn(:,:,enne) + E_bn(:,:,1) + PP'*squeeze(E_bn(:,:,enne-1))*PP ;  % Second order x
    
    for i_s = 1:size(hss,2)
        kbns(enne,i_s) = kbns(1,i_s) + kbns(enne-1,:)*PIMAT(:,i_s) + F_bn(:,enne-1)'*kxs(:,i_s) + ...
            1/2*vec(squeeze(eta(:,:,i_s)))'*vec(squeeze(E_bn(:,:,enne-1))) + ...
            1/2*F_bn(:,enne-1)'*squeeze(eta(:,:,i_s))*F_bn(:,enne-1) + ...
            F_bn(:,enne-1)'*squeeze(eta(:,:,i_s))*(Flam - Fpi);  % Second order sig
    end
end

% =========================================================================
%               Reshape matrices for squared terms as in the
%               representation used for estimation and select observables
% =========================================================================
%
% The Klein representation is
%   y = ybar + k_y + Fx + 1/2 kron(I,x')Ex
% while the representation used for estimation is
%   y = k_y + Fx + 1/2 E kron(x,x)
%
% FIRST ORDER MATRICES
% Select observable variables
gx = gx(n3,:);

% SECOND ORDER MATRICES IN X
newhxx = hxx ;
newgxx = gxx ;
hxx = zeros(n1_,n1_*n1_) ;
gxx = zeros(n2,n1_*n1_) ;
for i = 1:n2
    hxx(i,:) = (vec(newhxx(1+n1_*(i-1):n1_+n1_*(i-1),:)))' ;
    gxx(i,:) = (vec(newgxx(1+n1_*(i-1):n1_+n1_*(i-1),:)))' ;
end

% Select observable variables
gxx = gxx(n3,:);

% SECOND ORDER MATRICES IN sigma
% Note: the nonstochastic mean is added directly to gss. It is multiplied by 2
% because gss is divided by 2 in the reduced form representation
% Note also that the nonstochastic mean of the yield is identical to that
% of the short rate

% Select observable variables
%gss = [gss(n3,:) + 2*[Pi_t;deltaC;I_t;deltaY]*ones(1,size(hss,2))];
%gss = [gss(n3,:) + 2*[Pi_t;deltaC;deltaY;I_t]*ones(1,size(hss,2))];
% corrected 06/01/2020
%gss = gss(n3,:) + 2*y_(n3)*ones(1,size(hss,2));
gss = 2*gss(n3,:) + 2*y_(n3)*ones(1,size(hss,2));
% corrected 06/05/2020
hss = 2*hss;

% =========================================================================
%               Add bond yields
% =========================================================================
%
% Note that, by definition, bond yields (in deviation
% from the deterministic steady state) are
%   y_t,n = -(1/n)*b_t,n
% where b_t,n is the bond price
% Hence, all values of gx_nbp, gxx_nbp,  gss_nbp
% must be divided by enne
% Add the bonds to the obvservation matrices
for i = 1:size(n4,2) % Number of bonds used in estimation
    gx = [gx; -(1/n4(i))*F_bn(:,n4(i))'];
    gxx = [gxx; -(1/n4(i))*(vec(squeeze(E_bn(:,:,n4(i)))))'];
% corrected 06-01-2020
%   gss = [gss; -(1/n4(i))*kbns(n4(i),:) + 2*I_t*ones(1,size(hss,2))] ;
    gss = [gss; -(2/n4(i))*kbns(n4(i),:) + 2*I_t*ones(1,size(hss,2))] ;
end
%% this is to compute intercepts for long bonds in the linear case
% all set equal to intercept for shor term bond
nlb=size(n4,2); % Number of bonds used in estimation
if order==1
    gss(end-nlb+1:end,:)=repmat(gss(end-nlb,:),nlb,1);
end
%% here I report also the point around which the approximation is taken (NSSS)
%y_=[Pi_t;deltaC;deltaY;I_t*ones(nlb+1,1)];
%y_=[Pi_t;deltaC;I_t*ones(nlb+1,1)];
y_=[y_(n3);I_t*ones(nlb,1)];

%{
%% 04-27-2020, CORRECTED 06-04-2020
% rearrange matrices to "absorb" role of discrete state now 
% in the x vector
% x = [z;s], where z is (9 x 1) and s is (1 x 1)

% s = pi(1) or -pi(1), where pi is unconditional probability


% hx and hxx contain terms corresponding to columns corresponing
% to terms involving s.

% so 10_th colums of hx is an intercept shift that will be
% hx(:,10)*(1-pv(1)) or -hx(:,10)*pv(1);
% columns of hxx involving multiplications with s will be 
% state specific shifts to hx
% these columns are: 10 20 30 40 50 60 70 80 99 
% columns [1:9, 11:19, ..., 91:99] are genuine second order terms in z
% column 100 is an intercept shift

ndx_all=reshape(1:n1_^2,n1_,[]); % all columns
ndx_zz=reshape(ndx_all(1:end-1,1:end-1),[],1); % true quadratic terms in z
ndx_zs=ndx_all(end,1:end-1)'; % columns producing shifts in linear terms
ndx_sz=ndx_all(1:end-1,end) ; % columns producing shifts in linear terms
ndx_ss=ndx_all(end,end); % term generating intercept shift

% states 1-4: s_pist =1; 5-8: s_pist=0
%
%st_pist_t: unconditional probability of state 1
%
%trim last line off the h-matrices
hss=hss(1:n1,:);
hx=hx(1:n1,:);
hxx=hxx(1:n1,:);
% now, do the intercept adjustm
hss(:,1:4)=hss(:,1:4)+repmat((hxx(:,ndx_ss)*(1-st_pist_t)+2*hx(:,end))*(1-st_pist_t),1,4);
hss(:,5:8)=hss(:,5:8)+repmat((hxx(:,ndx_ss)*( -st_pist_t)+2*hx(:,end))*( -st_pist_t),1,4);
gss(:,1:4)=gss(:,1:4)+repmat((gxx(:,ndx_ss)*(1-st_pist_t)+2*gx(:,end))*(1-st_pist_t),1,4);
gss(:,5:8)=gss(:,5:8)+repmat((gxx(:,ndx_ss)*( -st_pist_t)+2*gx(:,end))*( -st_pist_t),1,4);
% trim last column of hx and gx
hx=hx(:,1:n1);
gx=gx(:,1:n1);
% adjustment on linear term
hx=repmat(hx,1,1,8);
gx=repmat(gx,1,1,8);
hx(:,:,1:4)=hx(:,:,1:4)+repmat((hxx(:,ndx_zs)+hxx(:,ndx_sz))*(1-st_pist_t),1,1,4)/2;
hx(:,:,5:8)=hx(:,:,5:8)+repmat((hxx(:,ndx_zs)+hxx(:,ndx_sz))*( -st_pist_t),1,1,4)/2;
gx(:,:,1:4)=gx(:,:,1:4)+repmat((gxx(:,ndx_zs)+gxx(:,ndx_sz))*(1-st_pist_t),1,1,4)/2;
gx(:,:,5:8)=gx(:,:,5:8)+repmat((gxx(:,ndx_zs)+gxx(:,ndx_sz))*( -st_pist_t),1,1,4)/2;
% keep only the needed second order terms
hxx=hxx(:,ndx_zz);
gxx=gxx(:,ndx_zz);
%
% get rid last row (and column) of eta_std (and eta)
eta=eta(1:n1,1:n1,:);
etastd=etastd(1:n1,:,:);
%}